from os import urandom
from typing import Dict

from litestar import Litestar, Request, delete, get, post
from litestar.middleware.session.client_side import CookieBackendConfig

# we initialize to config with a 16 byte key, i.e. 128 a bit key.
# in real world usage we should inject the secret from the environment
session_config = CookieBackendConfig(secret=urandom(16))  # type: ignore[arg-type]


@get("/session", sync_to_thread=False)
def check_session_handler(request: Request) -> Dict[str, bool]:
    """Handler function that accesses request.session."""
    return {"has_session": request.session != {}}


@post("/session", sync_to_thread=False)
def create_session_handler(request: Request) -> None:
    """Handler to set the session."""
    if not request.session:
        # value can be a dictionary or pydantic model
        request.set_session({"username": "moishezuchmir"})


@delete("/session", sync_to_thread=False)
def delete_session_handler(request: Request) -> None:
    """Handler to clear the session."""
    if request.session:
        request.clear_session()


app = Litestar(
    route_handlers=[check_session_handler, create_session_handler, delete_session_handler],
    middleware=[session_config.middleware],
)



from litestar import Litestar, Request, get


def key_builder(request: Request) -> str:
    return request.url.path + request.headers.get("my-header", "")


@get("/cached-path", cache=True, cache_key_builder=key_builder)
async def cached_handler() -> str:
    return "cached"


app = Litestar([cached_handler])



from typing import Any, Dict

from litestar import Litestar, get
from litestar.datastructures import State


@get("/", sync_to_thread=False)
def handler(state: State) -> Dict[str, Any]:
    return state.dict()


app = Litestar(route_handlers=[handler], state=State({"count": 100}))



from typing import Any, Dict

from prometheus_client import REGISTRY

from litestar import get
from litestar.plugins.prometheus import PrometheusMiddleware
from litestar.status_codes import HTTP_200_OK
from litestar.testing import TestClient


def clear_collectors() -> None:
    collectors = list(REGISTRY._collector_to_names.keys())
    for collector in collectors:
        REGISTRY.unregister(collector)

    PrometheusMiddleware._metrics = {}


def test_prometheus_exporter_with_extra_config_example() -> None:
    from docs.examples.plugins.prometheus.using_prometheus_exporter_with_extra_configs import app

    clear_collectors()

    @get("/test")
    def home() -> Dict[str, Any]:
        return {"hello": "world"}

    app.register(home)

    with TestClient(app) as client:
        client.get("/home")
        metrics_exporter_response = client.get("/custom-path")

        assert metrics_exporter_response.status_code == HTTP_200_OK
        metrics = metrics_exporter_response.content.decode()

        assert (
            """litestar_requests_in_progress{app_name="litestar-example",location="earth",method="GET",path="/custom-path",status_code="200",version_no="v2.0"} 1.0"""
            in metrics
        )



from dataclasses import dataclass
from datetime import datetime
from typing import Generic, List, TypeVar

from sqlalchemy.orm import Mapped

from litestar import Litestar, get
from litestar.dto import DTOConfig
from litestar.plugins.sqlalchemy import SQLAlchemyDTO

from .my_lib import Base

T = TypeVar("T")


@dataclass
class WithCount(Generic[T]):
    count: int
    data: List[T]


class User(Base):
    name: Mapped[str]
    password: Mapped[str]
    created_at: Mapped[datetime]


class UserDTO(SQLAlchemyDTO[User]):
    config = DTOConfig(exclude={"password", "created_at"})


@get("/users", dto=UserDTO, sync_to_thread=False)
def get_users() -> WithCount[User]:
    return WithCount(
        count=1,
        data=[
            User(
                id=1,
                name="Litestar User",
                password="xyz",
                created_at=datetime.now(),
            ),
        ],
    )


app = Litestar(route_handlers=[get_users])

# run: /users



from typing import Any, Dict

import litestar.status_codes
from litestar import Litestar, get


@get(
    "/resources",
    status_code=litestar.status_codes.HTTP_418_IM_A_TEAPOT,
    media_type="application/vnd.example.resource+json",
)
async def retrieve_resource() -> Dict[str, Any]:
    return {
        "title": "Server thinks it is a teapot",
        "type": "Server delusion",
        "status": litestar.status_codes.HTTP_418_IM_A_TEAPOT,
    }


app = Litestar(route_handlers=[retrieve_resource])



from contextlib import asynccontextmanager
from typing import AsyncGenerator, List, Optional

from sqlalchemy import select
from sqlalchemy.exc import IntegrityError, NoResultFound
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker, create_async_engine
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column

from litestar import Litestar, get, post, put
from litestar.datastructures import State
from litestar.exceptions import ClientException, NotFoundException
from litestar.plugins.sqlalchemy import SQLAlchemySerializationPlugin
from litestar.status_codes import HTTP_409_CONFLICT


class Base(DeclarativeBase): ...


class TodoItem(Base):
    __tablename__ = "todo_items"

    title: Mapped[str] = mapped_column(primary_key=True)
    done: Mapped[bool]


@asynccontextmanager
async def db_connection(app: Litestar) -> AsyncGenerator[None, None]:
    engine = getattr(app.state, "engine", None)
    if engine is None:
        engine = create_async_engine("sqlite+aiosqlite:///todo.sqlite", echo=True)
        app.state.engine = engine

    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.create_all)

    try:
        yield
    finally:
        await engine.dispose()


sessionmaker = async_sessionmaker(expire_on_commit=False)


async def provide_transaction(state: State) -> AsyncGenerator[AsyncSession, None]:
    async with sessionmaker(bind=state.engine) as session:
        try:
            async with session.begin():
                yield session
        except IntegrityError as exc:
            raise ClientException(
                status_code=HTTP_409_CONFLICT,
                detail=str(exc),
            ) from exc


async def get_todo_by_title(todo_name: str, session: AsyncSession) -> TodoItem:
    query = select(TodoItem).where(TodoItem.title == todo_name)
    result = await session.execute(query)
    try:
        return result.scalar_one()
    except NoResultFound as e:
        raise NotFoundException(detail=f"TODO {todo_name!r} not found") from e


async def get_todo_list(done: Optional[bool], session: AsyncSession) -> List[TodoItem]:
    query = select(TodoItem)
    if done is not None:
        query = query.where(TodoItem.done.is_(done))

    result = await session.execute(query)
    return list(result.scalars().all())


@get("/")
async def get_list(transaction: AsyncSession, done: Optional[bool] = None) -> List[TodoItem]:
    return await get_todo_list(done, transaction)


@post("/")
async def add_item(data: TodoItem, transaction: AsyncSession) -> TodoItem:
    transaction.add(data)
    return data


@put("/{item_title:str}")
async def update_item(item_title: str, data: TodoItem, transaction: AsyncSession) -> TodoItem:
    todo_item = await get_todo_by_title(item_title, transaction)
    todo_item.title = data.title
    todo_item.done = data.done
    return todo_item


app = Litestar(
    [get_list, add_item, update_item],
    dependencies={"transaction": provide_transaction},
    lifespan=[db_connection],
    plugins=[SQLAlchemySerializationPlugin()],
)



from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

if TYPE_CHECKING:
    from litestar_htmx import (
        HTMXDetails,
        HTMXRequest,
    )

__all__ = ("HTMXDetails", "HTMXRequest")


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        import litestar_htmx

        module = "litestar.plugins.htmx"
        value = globals()[attr_name] = getattr(litestar_htmx, attr_name)

        warn_deprecation(
            deprecated_name=f"litestar.contrib.htmx.request.{attr_name}",
            version="2.13",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.htmx.request' is deprecated, please import it from '{module}' instead",
        )

        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")



from pydantic import BaseModel
from typing_extensions import Annotated

from litestar import Litestar, get
from litestar.exceptions import NotAuthorizedException
from litestar.params import Parameter

USER_DB = {
    1: {
        "id": 1,
        "name": "John Doe",
    },
}

VALID_TOKEN = "super-secret-secret"
VALID_COOKIE_VALUE = "cookie-secret"


class User(BaseModel):
    id: int
    name: str


@get(path="/users/{user_id:int}/")
async def get_user(
    user_id: int,
    token: Annotated[str, Parameter(header="X-API-KEY")],
    cookie: Annotated[str, Parameter(cookie="my-cookie-param")],
) -> User:
    if token != VALID_TOKEN or cookie != VALID_COOKIE_VALUE:
        raise NotAuthorizedException
    return User.model_validate(USER_DB[user_id])


app = Litestar(route_handlers=[get_user])



from typing import Final, NoReturn

try:
    import rich_click as click
except ImportError:
    import click  # type: ignore[no-redef]

_SUGGEST_OPTION_POSSIBILITIES: Final = {
    "-V": ["command `version`"],
    "--version": ["command `version`"],
}


def suggest_option(error: click.NoSuchOption) -> NoReturn:
    if error.possibilities:
        raise error

    new_possibilities = _SUGGEST_OPTION_POSSIBILITIES.get(error.option_name)
    if new_possibilities is None:
        raise error

    new_error = click.NoSuchOption(
        option_name=error.option_name,
        possibilities=new_possibilities,
        message=error.message,
        ctx=error.ctx,
    )
    raise new_error from error



import anyio

from litestar import Litestar
from litestar.middleware import ASGIMiddleware
from litestar.types import ASGIApp, Receive, Scope, Send


class TimeoutMiddleware(ASGIMiddleware):
    def __init__(self, timeout: float):
        self.timeout = timeout

    async def handle(self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp) -> None:
        with anyio.move_on_after(self.timeout):
            await next_app(scope, receive, send)


app = Litestar(middleware=[TimeoutMiddleware(timeout=5)])



from __future__ import annotations

from dataclasses import dataclass
from uuid import UUID

from litestar.dto import DataclassDTO


@dataclass
class User:
    id: UUID
    name: str


UserDTO = DataclassDTO[User]
UserReturnDTO = DataclassDTO[User]



from pathlib import Path

from litestar import Litestar
from litestar.stores.file import FileStore

file_store = FileStore(Path("data"))


async def on_startup() -> None:
    await file_store.delete_expired()


app = Litestar(on_startup=[on_startup])



from typing import AsyncIterator

import uvicorn

from litestar import Litestar, get
from litestar.config.cors import CORSConfig
from litestar.response import ServerSentEvent, ServerSentEventMessage
from litestar.types import SSEData


@get("/test_sse_empty")
async def handler() -> ServerSentEvent:
    async def generate() -> AsyncIterator[SSEData]:
        event = ServerSentEventMessage(event="empty")
        yield event

    return ServerSentEvent(generate())


app = Litestar(route_handlers=[handler], cors_config=CORSConfig(allow_origins=["*"]))

if __name__ == "__main__":
    uvicorn.run("sse_empty:app")



from litestar.openapi.plugins import ScalarRenderPlugin

scalar_plugin = ScalarRenderPlugin(
    js_url="https://example.com/my-custom-scalar.js",
    css_url="https://example.com/my-custom-scalar.css",
    path="/scalar",
)



import os
from typing import cast

from sqlalchemy.ext.asyncio import AsyncEngine, create_async_engine

from litestar import Litestar

DB_URI = os.environ.get("DATABASE_URI", "postgresql+asyncpg://postgres:mysecretpassword@pg.db:5432/db")


def get_db_connection(app: Litestar) -> AsyncEngine:
    """Returns the db engine.

    If it doesn't exist, creates it and saves it in on the application state object
    """
    if not getattr(app.state, "engine", None):
        app.state.engine = create_async_engine(DB_URI)
    return cast("AsyncEngine", app.state.engine)


async def close_db_connection(app: Litestar) -> None:
    """Closes the db connection stored in the application State object."""
    if getattr(app.state, "engine", None):
        await cast("AsyncEngine", app.state.engine).dispose()


app = Litestar(on_startup=[get_db_connection], on_shutdown=[close_db_connection])



from __future__ import annotations

import re
from collections import defaultdict
from typing import TYPE_CHECKING, Any, AsyncGenerator

from multipart import (  # type: ignore[import-untyped]
    MultipartSegment,
    ParserError,
    ParserLimitReached,
    PushMultipartParser,
)

from litestar.datastructures.upload_file import UploadFile
from litestar.exceptions import ClientException

__all__ = ("parse_content_header", "parse_multipart_form")

from litestar.utils.compat import async_next

if TYPE_CHECKING:
    from litestar.types import TypeDecodersSequence

_token = r"([\w!#$%&'*+\-.^_`|~]+)"  # noqa: S105
_quoted = r'"([^"]*)"'
_param = re.compile(rf";\s*{_token}=(?:{_token}|{_quoted})", re.ASCII)
_firefox_quote_escape = re.compile(r'\\"(?!; |\s*$)')


def parse_content_header(value: str) -> tuple[str, dict[str, str]]:
    """Parse content-type and content-disposition header values.

    Args:
        value: A header string value to parse.

    Returns:
        A tuple containing the normalized header string and a dictionary of parameters.
    """
    value = _firefox_quote_escape.sub("%22", value)
    pos = value.find(";")
    if pos == -1:
        options: dict[str, str] = {}
    else:
        options = {
            m.group(1).lower(): m.group(2) or m.group(3).replace("%22", '"') for m in _param.finditer(value[pos:])
        }
        value = value[:pos]
    return value.strip().lower(), options


async def _close_upload_files(fields: dict[str, list[Any]]) -> None:
    for values in fields.values():
        for value in values:
            if isinstance(value, UploadFile):
                await value.close()


async def parse_multipart_form(  # noqa: C901
    stream: AsyncGenerator[bytes, None],
    boundary: bytes,
    multipart_form_part_limit: int = 1000,
    type_decoders: TypeDecodersSequence | None = None,
) -> dict[str, Any]:
    """Parse multipart form data.

    Args:
        stream: Body of the request.
        boundary: Boundary of the multipart message.
        multipart_form_part_limit: Limit of the number of parts allowed.
        type_decoders: A sequence of type decoders to use.

    Returns:
        A dictionary of parsed results.
    """

    fields: defaultdict[str, list[Any]] = defaultdict(list)

    chunk = await async_next(stream, b"")
    if not chunk:
        return fields

    data: UploadFile | bytearray = bytearray()

    try:
        with PushMultipartParser(boundary, max_segment_count=multipart_form_part_limit) as parser:
            segment: MultipartSegment | None = None
            while not parser.closed:
                for form_part in parser.parse(chunk):
                    if isinstance(form_part, MultipartSegment):
                        segment = form_part
                        if segment.filename:
                            data = UploadFile(
                                content_type=segment.content_type or "text/plain",
                                filename=segment.filename,
                                headers=dict(segment.headerlist),
                            )
                    elif form_part:
                        if isinstance(data, UploadFile):
                            await data.write(form_part)
                        else:
                            data.extend(form_part)
                    else:
                        # end of part
                        if segment is None:
                            # we have reached the end of a segment before we have
                            # received a complete header segment
                            raise ClientException("Unexpected eof in multipart/form-data")

                        if isinstance(data, UploadFile):
                            await data.seek(0)
                            fields[segment.name].append(data)
                        else:
                            # Always decode as string, even for empty data
                            fields[segment.name].append(data.decode(segment.charset or "utf-8"))

                        # reset for next part
                        data = bytearray()
                        segment = None

                chunk = await async_next(stream, b"")

    except ParserError as exc:
        # if an exception is raised, make sure that all 'UploadFile's are closed
        if isinstance(data, UploadFile):
            await data.close()
        await _close_upload_files(fields)

        raise ClientException("Invalid multipart/form-data") from exc
    except ParserLimitReached:
        if isinstance(data, UploadFile):
            await data.close()
        await _close_upload_files(fields)

        # FIXME (3.0): This should raise a '413 - Request Entity Too Large', but for
        # backwards compatibility, we keep it as a 400 for now
        raise ClientException("Request Entity Too Large") from None

    return {k: v if len(v) > 1 else v[0] for k, v in fields.items()}



from __future__ import annotations

import itertools
import json
import pathlib
import re
import sys

__all__ = (
    "find_resolved_issues",
    "main",
)


def find_resolved_issues(source: str, tag: str) -> list[str]:
    version = tag.split("v", maxsplit=1)[-1]
    changelog_line = f".. changelog:: {version}"
    stop_line = ".. changelog::"
    return list(
        {
            issue
            for line in itertools.takewhile(
                lambda l: stop_line not in l,  # noqa: E741
                source.split(changelog_line, maxsplit=1)[1].splitlines(),
            )
            if re.match(r"\s+:issue: [\d ,]+", line)
            for issue in re.findall(r"\d+", line)
        }
    )


def main(tag: str) -> str:
    source = pathlib.Path("docs/release-notes/changelog.rst").read_text()
    return json.dumps(find_resolved_issues(source, tag))


if __name__ == "__main__":
    print(main(sys.argv[1]))  # noqa: T201



from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

__all__ = ("EngineConfig",)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        from advanced_alchemy.extensions.litestar import EngineConfig

        module = "litestar.plugins.sqlalchemy"

        warn_deprecation(
            deprecated_name=f"litestar.contrib.sqlalchemy.plugins.init.config.engine.{attr_name}",
            version="2.12",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.sqlalchemy.plugins.init.config.engine' is deprecated, please "
            f"import it from '{module}' instead",
        )
        value = globals()[attr_name] = EngineConfig
        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")  # pragma: no cover


if TYPE_CHECKING:
    from advanced_alchemy.extensions.litestar import EngineConfig



from litestar import Litestar
from litestar.static_files import create_static_files_router

app = Litestar(
    route_handlers=[
        create_static_files_router(
            path="/",
            directories=["assets"],
            opt={"some": True},
            include_in_schema=False,
            tags=["static"],
        )
    ]
)



from types import ModuleType, TracebackType
from typing import Any, Optional, Protocol, Union

from typing_extensions import TypeAlias


class PDBProtocol(Protocol):
    @staticmethod
    def post_mortem(
        traceback: Optional[TracebackType] = None,
        *args: Any,
        **kwargs: Any,
    ) -> Any: ...


Debugger: TypeAlias = Union[ModuleType, PDBProtocol]



from dataclasses import dataclass
from typing import Any

import anyio

from litestar import Litestar, MediaType, Request, Response, WebSocket, get, websocket
from litestar.connection import ASGIConnection
from litestar.datastructures import State
from litestar.di import Provide
from litestar.exceptions import NotAuthorizedException, NotFoundException
from litestar.middleware import AbstractAuthenticationMiddleware, AuthenticationResult
from litestar.middleware.base import DefineMiddleware

API_KEY_HEADER = "X-API-KEY"

TOKEN_USER_DATABASE = {"1": "user_authorized"}


@dataclass
class MyUser:
    name: str


@dataclass
class MyToken:
    api_key: str


class CustomAuthenticationMiddleware(AbstractAuthenticationMiddleware):
    async def authenticate_request(self, connection: ASGIConnection) -> AuthenticationResult:
        """Given a request, parse the request api key stored in the header and retrieve the user correlating to the token from the DB"""

        # retrieve the auth header
        auth_header = connection.headers.get(API_KEY_HEADER)
        if not auth_header:
            raise NotAuthorizedException()

        # this would be a database call
        token = MyToken(api_key=auth_header)
        user = MyUser(name=TOKEN_USER_DATABASE.get(token.api_key))
        if not user.name:
            raise NotAuthorizedException()
        return AuthenticationResult(user=user, auth=token)


@get("/")
def my_http_handler(request: Request[MyUser, MyToken, State]) -> None:
    user = request.user  # correctly typed as MyUser
    auth = request.auth  # correctly typed as MyToken
    assert isinstance(user, MyUser)
    assert isinstance(auth, MyToken)


@websocket("/")
async def my_ws_handler(socket: WebSocket[MyUser, MyToken, State]) -> None:
    user = socket.user  # correctly typed as MyUser
    auth = socket.auth  # correctly typed as MyToken
    assert isinstance(user, MyUser)
    assert isinstance(auth, MyToken)


@get(path="/", exclude_from_auth=True)
async def site_index() -> Response:
    """Site index"""
    exists = await anyio.Path("index.html").exists()
    if exists:
        async with await anyio.open_file(anyio.Path("index.html")) as file:
            content = await file.read()
            return Response(content=content, status_code=200, media_type=MediaType.HTML)
    raise NotFoundException("Site index was not found")


async def my_dependency(request: Request[MyUser, MyToken, State]) -> Any:
    user = request.user  # correctly typed as MyUser
    auth = request.auth  # correctly typed as MyToken
    assert isinstance(user, MyUser)
    assert isinstance(auth, MyToken)


# you can optionally exclude certain paths from authentication.
# the following excludes all routes mounted at or under `/schema*`
auth_mw = DefineMiddleware(CustomAuthenticationMiddleware, exclude="schema")

app = Litestar(
    route_handlers=[site_index, my_http_handler, my_ws_handler],
    middleware=[auth_mw],
    dependencies={"some_dependency": Provide(my_dependency)},
)



from __future__ import annotations

from typing import TYPE_CHECKING, Sequence

from sqlalchemy import select
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column

from litestar import Litestar, post
from litestar.plugins.sqlalchemy import SQLAlchemyAsyncConfig, SQLAlchemyPlugin

if TYPE_CHECKING:
    from sqlalchemy.ext.asyncio import AsyncSession


class Base(DeclarativeBase): ...


class TodoItem(Base):
    __tablename__ = "todo_item"
    title: Mapped[str] = mapped_column(primary_key=True)
    done: Mapped[bool]


@post("/")
async def add_item(data: TodoItem, db_session: AsyncSession) -> Sequence[TodoItem]:
    async with db_session.begin():
        db_session.add(data)
    return (await db_session.execute(select(TodoItem))).scalars().all()


config = SQLAlchemyAsyncConfig(
    connection_string="sqlite+aiosqlite:///todo_async.sqlite", create_all=True, metadata=Base.metadata
)
plugin = SQLAlchemyPlugin(config=config)
app = Litestar(route_handlers=[add_item], plugins=[plugin])



# ruff: noqa: F401
# pyright: reportUnusedImport=false
from __future__ import annotations

import importlib
import sys
from pathlib import Path

import pytest
from advanced_alchemy import exceptions as advanced_alchemy_exceptions
from advanced_alchemy import repository as advanced_alchemy_repo
from advanced_alchemy import types as advanced_alchemy_types
from advanced_alchemy.repository import typing as advanced_alchemy_typing
from sqlalchemy import Engine, create_engine
from sqlalchemy.ext.asyncio import AsyncEngine, create_async_engine


def purge_module(module_names: list[str], path: str | Path) -> None:
    for name in module_names:
        if name in sys.modules:
            del sys.modules[name]
    Path(importlib.util.cache_from_source(path)).unlink(missing_ok=True)  # type: ignore[arg-type]


def test_create_engine_with_engine_instance() -> None:
    from litestar.contrib.sqlalchemy.plugins.init.config.sync import SQLAlchemySyncConfig

    engine = create_engine("sqlite:///:memory:")
    config = SQLAlchemySyncConfig(engine_instance=engine)
    with pytest.deprecated_call():
        assert engine is config.create_engine()  # type: ignore[attr-defined]


def test_create_engine_with_connection_string() -> None:
    from litestar.contrib.sqlalchemy.plugins.init.config.sync import SQLAlchemySyncConfig

    config = SQLAlchemySyncConfig(connection_string="sqlite:///:memory:")
    with pytest.deprecated_call():
        engine = config.create_engine()  # type: ignore[attr-defined]
    assert isinstance(engine, Engine)


def test_async_create_engine_with_engine_instance() -> None:
    from litestar.contrib.sqlalchemy.plugins.init.config.asyncio import SQLAlchemyAsyncConfig

    engine = create_async_engine("sqlite+aiosqlite:///:memory:")
    config = SQLAlchemyAsyncConfig(engine_instance=engine)
    with pytest.deprecated_call():
        assert engine is config.create_engine()  # type: ignore[attr-defined]


def test_async_create_engine_with_connection_string() -> None:
    from litestar.contrib.sqlalchemy.plugins.init.config.asyncio import SQLAlchemyAsyncConfig

    config = SQLAlchemyAsyncConfig(connection_string="sqlite+aiosqlite:///:memory:")
    with pytest.deprecated_call():
        engine = config.create_engine()  # type: ignore[attr-defined]
    assert isinstance(engine, AsyncEngine)


def test_repository_re_exports() -> None:
    from litestar.contrib.sqlalchemy import types
    from litestar.contrib.sqlalchemy.repository import (
        SQLAlchemyAsyncRepository,
        SQLAlchemySyncRepository,
        wrap_sqlalchemy_exception,
    )
    from litestar.contrib.sqlalchemy.repository import types as repository_types

    assert wrap_sqlalchemy_exception is advanced_alchemy_exceptions.wrap_sqlalchemy_exception
    assert SQLAlchemySyncRepository is advanced_alchemy_repo.SQLAlchemySyncRepository
    assert SQLAlchemyAsyncRepository is advanced_alchemy_repo.SQLAlchemyAsyncRepository
    assert repository_types.ModelT is advanced_alchemy_typing.ModelT  # pyright: ignore[reportGeneralTypeIssues]
    assert repository_types.RowT is advanced_alchemy_typing.RowT  # pyright: ignore[reportGeneralTypeIssues]
    assert repository_types.SQLAlchemyAsyncRepositoryT is advanced_alchemy_typing.SQLAlchemyAsyncRepositoryT  # pyright: ignore[reportGeneralTypeIssues]
    assert repository_types.SQLAlchemySyncRepositoryT is advanced_alchemy_typing.SQLAlchemySyncRepositoryT  # pyright: ignore[reportGeneralTypeIssues]

    assert types.GUID is advanced_alchemy_types.GUID
    assert types.ORA_JSONB is advanced_alchemy_types.ORA_JSONB
    assert types.BigIntIdentity is advanced_alchemy_types.BigIntIdentity
    assert types.DateTimeUTC is advanced_alchemy_types.DateTimeUTC
    assert types.JsonB is advanced_alchemy_types.JsonB


def test_deprecated_sqlalchemy_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing SQLAlchemyAsyncRepository from 'litestar.contrib.sqlalchemy' is deprecated"
    ):
        from litestar.contrib.sqlalchemy import SQLAlchemyAsyncRepository
    purge_module(["litestar.contrib.sqlalchemy"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing SQLAlchemySyncRepository from 'litestar.contrib.sqlalchemy' is deprecated"
    ):
        from litestar.contrib.sqlalchemy import SQLAlchemySyncRepository
    purge_module(["litestar.contrib.sqlalchemy"], __file__)
    with pytest.warns(DeprecationWarning, match="importing ModelT from 'litestar.contrib.sqlalchemy' is deprecated"):
        from litestar.contrib.sqlalchemy import ModelT
    purge_module(["litestar.contrib.sqlalchemy"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing wrap_sqlalchemy_exception from 'litestar.contrib.sqlalchemy' is deprecated"
    ):
        from litestar.contrib.sqlalchemy import wrap_sqlalchemy_exception


def test_deprecated_sqlalchemy_plugins_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing AsyncSessionConfig from 'litestar.contrib.sqlalchemy.plugins' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins import AsyncSessionConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing EngineConfig from 'litestar.contrib.sqlalchemy.plugins' is deprecated"
    ):
        from litestar.contrib.sqlalchemy.plugins import EngineConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing GenericSQLAlchemyConfig from 'litestar.contrib.sqlalchemy.plugins' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins import GenericSQLAlchemyConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyAsyncConfig from 'litestar.contrib.sqlalchemy.plugins' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins import SQLAlchemyAsyncConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyInitPlugin from 'litestar.contrib.sqlalchemy.plugins' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins import SQLAlchemyInitPlugin


def test_deprecated_sqlalchemy_plugins_init_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing AsyncSessionConfig from 'litestar.contrib.sqlalchemy.plugins.init' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init import AsyncSessionConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing EngineConfig from 'litestar.contrib.sqlalchemy.plugins.init' is deprecated"
    ):
        from litestar.contrib.sqlalchemy.plugins.init import EngineConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing GenericSQLAlchemyConfig from 'litestar.contrib.sqlalchemy.plugins.init' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init import GenericSQLAlchemyConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyAsyncConfig from 'litestar.contrib.sqlalchemy.plugins.init' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init import SQLAlchemyAsyncConfig


def test_deprecated_sqlalchemy_plugins_init_config_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing AsyncSessionConfig from 'litestar.contrib.sqlalchemy.plugins.init.config' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config import AsyncSessionConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing EngineConfig from 'litestar.contrib.sqlalchemy.plugins.init.config' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config import EngineConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing GenericSQLAlchemyConfig from 'litestar.contrib.sqlalchemy.plugins.init.config' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config import GenericSQLAlchemyConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyAsyncConfig from 'litestar.contrib.sqlalchemy.plugins.init.config' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config import SQLAlchemyAsyncConfig


def test_deprecated_sqlalchemy_plugins_init_config_common_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.common"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SESSION_SCOPE_KEY from 'litestar.contrib.sqlalchemy.plugins.init.config.common' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.common import SESSION_SCOPE_KEY
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.common"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SESSION_TERMINUS_ASGI_EVENTS from 'litestar.contrib.sqlalchemy.plugins.init.config.common' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.common import SESSION_TERMINUS_ASGI_EVENTS
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.common"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing GenericSQLAlchemyConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.common' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.common import GenericSQLAlchemyConfig


def test_deprecated_sqlalchemy_plugins_init_config_sync_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.sync"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemySyncConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.sync' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.sync import SQLAlchemySyncConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.sync"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing AlembicSyncConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.sync' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.sync import AlembicSyncConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.sync"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SyncSessionConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.sync' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.sync import SyncSessionConfig


def test_deprecated_sqlalchemy_plugins_init_config_asyncio_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.asyncio"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyAsyncConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.asyncio' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.asyncio import SQLAlchemyAsyncConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.asyncio"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing AlembicAsyncConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.asyncio' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.asyncio import AlembicAsyncConfig
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.asyncio"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing AsyncSessionConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.asyncio' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.asyncio import AsyncSessionConfig


def test_deprecated_sqlalchemy_plugins_init_config_engine_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.engine"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing EngineConfig from 'litestar.contrib.sqlalchemy.plugins.init.config.engine' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.engine import EngineConfig


def test_deprecated_sqlalchemy_dto_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.dto"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyDTOConfig from 'litestar.contrib.sqlalchemy.dto' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.dto import SQLAlchemyDTOConfig


def test_deprecated_sqlalchemy_plugins_init_plugin_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.plugin"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyInitPlugin from 'litestar.contrib.sqlalchemy.plugins.init' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.plugin import SQLAlchemyInitPlugin


def test_deprecated_sqlalchemy_plugins_serialization_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.serialization"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemySerializationPlugin from 'litestar.contrib.sqlalchemy.plugins.serialization' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.serialization import SQLAlchemySerializationPlugin


def test_deprecated_sqlalchemy_repository_async_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.repository._async"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemyAsyncRepository from 'litestar.contrib.sqlalchemy.repository._async' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.repository._async import SQLAlchemyAsyncRepository


def test_deprecated_sqlalchemy_repository_sync_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.repository._sync"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing SQLAlchemySyncRepository from 'litestar.contrib.sqlalchemy.repository._sync' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.repository._sync import SQLAlchemySyncRepository


def test_deprecated_sqlalchemy_base_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.base"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="from 'litestar.contrib.sqlalchemy.base' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.base import (
            AuditColumns,
            BigIntAuditBase,
            BigIntBase,
            BigIntPrimaryKey,
            CommonTableAttributes,
            ModelProtocol,
            UUIDAuditBase,
            UUIDBase,
            UUIDPrimaryKey,
            create_registry,
            orm_registry,
            touch_updated_timestamp,
        )


def test_deprecated_sqlalchemy_plugins_init_config_asyncio_handlers() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.asyncio"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing default_before_send_handler from 'litestar.contrib.sqlalchemy.plugins.init.config.asyncio' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.asyncio import default_before_send_handler

    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.asyncio"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing autocommit_before_send_handler from 'litestar.contrib.sqlalchemy.plugins.init.config.asyncio' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.asyncio import autocommit_before_send_handler


def test_deprecated_sqlalchemy_plugins_init_config_sync_handlers() -> None:
    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.sync"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing default_before_send_handler from 'litestar.contrib.sqlalchemy.plugins.init.config.sync' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.sync import default_before_send_handler

    purge_module(["litestar.contrib.sqlalchemy.plugins.init.config.sync"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing autocommit_before_send_handler from 'litestar.contrib.sqlalchemy.plugins.init.config.sync' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.plugins.init.config.sync import autocommit_before_send_handler


def test_deprecated_sqlalchemy_repository_util_imports() -> None:
    purge_module(["litestar.contrib.sqlalchemy.repository._util"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing wrap_sqlalchemy_exception from 'litestar.contrib.sqlalchemy.repository._util' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.repository._util import wrap_sqlalchemy_exception

    purge_module(["litestar.contrib.sqlalchemy.repository._util"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing get_instrumented_attr from 'litestar.contrib.sqlalchemy.repository._util' is deprecated",
    ):
        from litestar.contrib.sqlalchemy.repository._util import get_instrumented_attr



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Mapping

from litestar.utils.helpers import get_name

if TYPE_CHECKING:
    from collections.abc import Sequence

    from litestar.openapi.spec import Example
    from litestar.typing import FieldDefinition

__all__ = ("_should_create_literal_schema",)


def _should_create_literal_schema(field_definition: FieldDefinition) -> bool:
    """Predicate to determine if we should create a literal schema for the field def, or not.

    This returns ``True`` if the field definition is an literal, or if the field definition is a union
    of a literal and None.

    When an annotation is `Literal["anything"] | None` we should create a schema for the literal that includes `null`
    in the enum values.

    Args:
        field_definition: A field definition instance.

    Returns:
        A boolean
    """
    return field_definition.is_literal or (
        field_definition.is_optional
        and all(inner.is_literal for inner in field_definition.inner_types if not inner.is_none_type)
    )


def get_example_id(example: Example, name: str, index: int) -> str:
    """Get the example ID.

    Args:
        example: The example instance.
        name: The name of the field.
        index: The index of the example.

    Returns:
        The example ID.
    """
    return example.id or f"{name}-example-{index}"


def get_formatted_examples(field_definition: FieldDefinition, examples: Sequence[Example]) -> Mapping[str, Example]:
    """Format the examples into the OpenAPI schema format."""

    name = field_definition.name or get_name(field_definition.type_)
    name = name.lower()

    return {get_example_id(example, name, i): example for i, example in enumerate(examples, 1)}


def get_json_schema_formatted_examples(examples: Sequence[Example]) -> list[Any]:
    """Format the examples into the JSON schema format."""
    return [example.value for example in examples]



from __future__ import annotations

from dataclasses import dataclass

from litestar import Litestar, post
from litestar.plugins.problem_details import ProblemDetailsConfig, ProblemDetailsException, ProblemDetailsPlugin


@dataclass
class PurchaseItem:
    item_id: int
    quantity: int


class PurchaseNotAllowedError(Exception):
    def __init__(self, account_id: int, balance: int, detail: str) -> None:
        self.account_id = account_id
        self.balance = balance
        self.detail = detail


@post("/purchase")
async def purchase(data: PurchaseItem) -> None:
    raise PurchaseNotAllowedError(
        account_id=12345,
        balance=30,
        detail="Your current balance is 30, but that costs 50.",
    )


def convert_purchase_not_allowed_to_problem_details(exc: PurchaseNotAllowedError) -> ProblemDetailsException:
    return ProblemDetailsException(
        type_="https://example.com/probs/out-of-credit",
        title="You do not have enough credit.",
        detail=exc.detail,
        instance=f"/account/{exc.account_id}/msgs/abc",
        extra={"balance": exc.balance},
    )


problem_details_plugin = ProblemDetailsPlugin(
    ProblemDetailsConfig(
        enable_for_all_http_exceptions=True,
        exception_to_problem_detail_map={PurchaseNotAllowedError: convert_purchase_not_allowed_to_problem_details},
    )
)
app = Litestar(route_handlers=[purchase], plugins=[problem_details_plugin])

# run: /purchase --header "Content-Type: application/json" --request POST --data '{"item_id": 1234, "quantity": 2}'



from litestar import Litestar, Request, get
from litestar.contrib.jinja import JinjaTemplateEngine
from litestar.middleware.session.server_side import ServerSideSessionConfig
from litestar.plugins.flash import FlashConfig, FlashPlugin, flash
from litestar.response import Template
from litestar.template.config import TemplateConfig

template_config = TemplateConfig(engine=JinjaTemplateEngine, directory="templates")
flash_plugin = FlashPlugin(config=FlashConfig(template_config=template_config))


@get()
async def index(request: Request) -> Template:
    """Example of adding and displaying a flash message."""
    flash(request, "Oh no! I've been flashed!", category="error")

    return Template(
        template_str="""
    <h1>Flash Message Example</h1>
    {% for message in get_flashes() %}
    <p>{{ message.message }} (Category:{{ message.category }})</p>
    {% endfor %}
    """
    )


app = Litestar(
    plugins=[flash_plugin],
    route_handlers=[index],
    template_config=template_config,
    middleware=[ServerSideSessionConfig().middleware],
)



from litestar import Litestar, WebSocket, websocket
from litestar.channels import ChannelsPlugin
from litestar.channels.backends.memory import MemoryChannelsBackend


@websocket("/ws")
async def handler(socket: WebSocket, channels: ChannelsPlugin) -> None:
    await socket.accept()

    async with channels.start_subscription(["some_channel"]) as subscriber:
        async for message in subscriber.iter_events():
            await socket.send_text(message)


app = Litestar(
    [handler],
    plugins=[ChannelsPlugin(backend=MemoryChannelsBackend())],
)



import dataclasses
import secrets
from typing import Any, Dict

from litestar import Litestar, Request, get
from litestar.connection import ASGIConnection
from litestar.security.jwt import JWTAuth, Token


@dataclasses.dataclass
class CustomToken(Token):
    token_flag: bool = False


@dataclasses.dataclass
class User:
    id: str


async def retrieve_user_handler(token: CustomToken, connection: ASGIConnection) -> User:
    return User(id=token.sub)


TOKEN_SECRET = secrets.token_hex()

jwt_auth = JWTAuth[User](
    token_secret=TOKEN_SECRET,
    retrieve_user_handler=retrieve_user_handler,
    token_cls=CustomToken,
)


@get("/")
def handler(request: Request[User, CustomToken, Any]) -> Dict[str, Any]:
    return {"id": request.user.id, "token_flag": request.auth.token_flag}


app = Litestar(middleware=[jwt_auth.middleware])



from __future__ import annotations

import dataclasses
import inspect
from inspect import Signature
from typing import Any

import msgspec

from litestar.openapi.spec import Example
from litestar.params import ParameterKwarg
from litestar.plugins import DIPlugin

__all__ = ("MsgspecDIPlugin", "kwarg_definition_from_field")


class MsgspecDIPlugin(DIPlugin):
    def has_typed_init(self, type_: Any) -> bool:
        return type(type_) is type(msgspec.Struct)

    def get_typed_init(self, type_: Any) -> tuple[Signature, dict[str, Any]]:
        parameters = []
        type_hints = {}
        for field_info in msgspec.structs.fields(type_):
            type_hints[field_info.name] = field_info.type
            parameters.append(
                inspect.Parameter(
                    name=field_info.name,
                    kind=inspect.Parameter.KEYWORD_ONLY,
                    annotation=field_info.type,
                    default=field_info.default,
                )
            )
        return inspect.Signature(parameters), type_hints


def kwarg_definition_from_field(field: msgspec.inspect.Field) -> tuple[ParameterKwarg | None, dict[str, Any]]:
    extra: dict[str, Any] = {}
    kwargs: dict[str, Any] = {}
    if isinstance(field.type, msgspec.inspect.Metadata):
        meta = field.type
        field_type = meta.type
        if extra_json_schema := meta.extra_json_schema:
            kwargs["title"] = extra_json_schema.get("title")
            kwargs["description"] = extra_json_schema.get("description")
            if examples := extra_json_schema.get("examples"):
                kwargs["examples"] = [Example(value=e) for e in examples]
            kwargs["schema_extra"] = extra_json_schema.get("extra")
        extra = meta.extra or {}
    else:
        field_type = field.type

    if isinstance(
        field_type,
        (
            msgspec.inspect.IntType,
            msgspec.inspect.FloatType,
        ),
    ):
        kwargs["gt"] = field_type.gt
        kwargs["ge"] = field_type.ge
        kwargs["lt"] = field_type.lt
        kwargs["le"] = field_type.le
        kwargs["multiple_of"] = field_type.multiple_of
    elif isinstance(
        field_type,
        (
            msgspec.inspect.StrType,
            msgspec.inspect.BytesType,
            msgspec.inspect.ByteArrayType,
            msgspec.inspect.MemoryViewType,
        ),
    ):
        kwargs["min_length"] = field_type.min_length
        kwargs["max_length"] = field_type.max_length
        if isinstance(field_type, msgspec.inspect.StrType):
            kwargs["pattern"] = field_type.pattern

    parameter_defaults = {
        f.name: default for f in dataclasses.fields(ParameterKwarg) if (default := f.default) is not dataclasses.MISSING
    }
    kwargs_without_defaults = {k: v for k, v in kwargs.items() if v != parameter_defaults[k]}

    if kwargs_without_defaults:
        return ParameterKwarg(**kwargs_without_defaults), extra
    return None, extra



from asyncio import sleep
from datetime import datetime
from typing import AsyncGenerator

from litestar import Litestar, get
from litestar.response import Stream
from litestar.serialization import encode_json


async def my_generator() -> AsyncGenerator[bytes, None]:
    while True:
        await sleep(0.01)
        yield encode_json({"current_time": datetime.now()})


@get(path="/time")
def stream_time() -> Stream:
    return Stream(my_generator())


app = Litestar(route_handlers=[stream_time])



from litestar import Litestar, get
from litestar.logging.config import LoggingConfig
from litestar.middleware.logging import LoggingMiddlewareConfig


@get("/")
async def handler() -> dict[str, str]:
    return {"hello": "world"}


logging_middleware_config = LoggingMiddlewareConfig()

app = Litestar(
    route_handlers=[handler],
    logging_config=LoggingConfig(),
    middleware=[logging_middleware_config.middleware],
)


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app)



from __future__ import annotations

import uuid
from datetime import date
from typing import List
from uuid import UUID

from sqlalchemy import ForeignKey, func, select
from sqlalchemy.ext.asyncio import AsyncEngine, AsyncSession
from sqlalchemy.orm import Mapped, mapped_column, relationship

from litestar import Litestar, get
from litestar.plugins.sqlalchemy import AsyncSessionConfig, SQLAlchemyAsyncConfig, SQLAlchemyPlugin, base


# The SQLAlchemy base includes a declarative model for you to use in your models.
# The `UUIDBase` class includes a `UUID` based primary key (`id`)
class Author(base.UUIDBase):
    __tablename__ = "author"
    name: Mapped[str]
    dob: Mapped[date]
    books: Mapped[List[Book]] = relationship(back_populates="author", lazy="selectin")


# The `UUIDAuditBase` class includes the same UUID` based primary key (`id`) and 2
# additional columns: `created_at` and `updated_at`. `created_at` is a timestamp of when the
# record created, and `updated_at` is the last time the record was modified.
class Book(base.UUIDAuditBase):
    __tablename__ = "book"
    title: Mapped[str]
    author_id: Mapped[UUID] = mapped_column(ForeignKey("author.id"))
    author: Mapped[Author] = relationship(lazy="joined", innerjoin=True, viewonly=True)


session_config = AsyncSessionConfig(expire_on_commit=False)
sqlalchemy_config = SQLAlchemyAsyncConfig(
    connection_string="sqlite+aiosqlite:///test.sqlite", session_config=session_config, create_all=True
)  # Create 'async_session' dependency.


async def on_startup(app: Litestar) -> None:
    """Adds some dummy data if no data is present."""
    async with sqlalchemy_config.get_session() as session:
        statement = select(func.count()).select_from(Author)
        count = await session.execute(statement)
        if not count.scalar():
            author_id = uuid.uuid4()
            session.add(Author(name="Stephen King", dob=date(1954, 9, 21), id=author_id))
            session.add(Book(title="It", author_id=author_id))
            await session.commit()


@get(path="/authors")
async def get_authors(db_session: AsyncSession, db_engine: AsyncEngine) -> List[Author]:
    """Interact with SQLAlchemy engine and session."""
    return list(await db_session.scalars(select(Author)))


app = Litestar(
    route_handlers=[get_authors],
    on_startup=[on_startup],
    debug=True,
    plugins=[SQLAlchemyPlugin(config=sqlalchemy_config)],
)



from litestar import Litestar
from litestar.stores.redis import RedisStore

root_store = RedisStore.with_client()
cache_store = root_store.with_namespace("cache")
session_store = root_store.with_namespace("sessions")


async def before_shutdown() -> None:
    await cache_store.delete_all()


app = Litestar(before_shutdown=[before_shutdown])



from __future__ import annotations

import importlib.metadata
import os
import re
import warnings
from functools import partial
from typing import Any

from sphinx.addnodes import document
from sphinx.application import Sphinx
from sqlalchemy.exc import SAWarning

warnings.filterwarnings("ignore", category=SAWarning)

__all__ = ["setup", "update_html_context"]

PY_CLASS = "py:class"
PY_RE = r"py:.*"
PY_METH = "py:meth"
PY_ATTR = "py:attr"
PY_OBJ = "py:obj"
PY_FUNC = "py:func"

project = "Litestar"
copyright = "2025, Litestar-Org"
author = "Litestar-Org"
release = os.getenv("_LITESTAR_DOCS_BUILD_VERSION", importlib.metadata.version("litestar").rsplit(".")[0])

extensions = [
    "sphinx.ext.intersphinx",
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx.ext.autosectionlabel",
    "sphinx_design",
    "auto_pytabs.sphinx_ext",
    "tools.sphinx_ext",
    "sphinx_copybutton",
    "sphinxcontrib.mermaid",
    "sphinx_click",
    "sphinx_paramlinks",
]

exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "msgspec": ("https://jcristharif.com/msgspec/", None),
    "anyio": ("https://anyio.readthedocs.io/en/stable/", None),
    "multidict": ("https://multidict.aio-libs.org/en/stable/", None),
    "cryptography": ("https://cryptography.io/en/latest/", None),
    "sqlalchemy": ("https://docs.sqlalchemy.org/en/20/", None),
    "alembic": ("https://alembic.sqlalchemy.org/en/latest/", None),
    "click": ("https://click.palletsprojects.com/en/8.1.x/", None),
    "redis": ("https://redis.readthedocs.io/en/stable/", None),
    "picologging": ("https://microsoft.github.io/picologging", None),
    "structlog": ("https://www.structlog.org/en/stable/", None),
    "tortoise": ("https://tortoise.github.io/", None),
    "piccolo": ("https://piccolo-orm.readthedocs.io/en/latest", None),
    "opentelemetry": ("https://opentelemetry-python.readthedocs.io/en/latest/", None),
    "advanced-alchemy": ("https://advanced-alchemy.litestar.dev/latest/", None),
    "jinja2": ("https://jinja.palletsprojects.com/en/latest/", None),
    "trio": ("https://trio.readthedocs.io/en/stable/", None),
    "pydantic": ("https://docs.pydantic.dev/latest/", None),
    "typing_extensions": ("https://typing-extensions.readthedocs.io/en/stable/", None),
    "valkey": ("https://valkey-py.readthedocs.io/en/latest/", None),
}

napoleon_google_docstring = True
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = True
napoleon_use_admonition_for_notes = True
napoleon_use_admonition_for_references = False
napoleon_attr_annotations = True

autoclass_content = "class"
autodoc_class_signature = "separated"
autodoc_default_options = {"special-members": "__init__", "show-inheritance": True, "members": True}
autodoc_member_order = "bysource"
autodoc_typehints_format = "short"
autodoc_mock_imports = []

nitpicky = True
nitpick_ignore = [
    # external library / undocumented external
    (PY_CLASS, "BaseModel"),
    (PY_CLASS, "ExternalType"),
    (PY_CLASS, "TypeEngine"),
    (PY_CLASS, "UserDefinedType"),
    (PY_CLASS, "_RegistryType"),
    (PY_CLASS, "_orm.Mapper"),
    (PY_CLASS, "_orm.registry"),
    (PY_CLASS, "_schema.MetaData"),
    (PY_CLASS, "_schema.Table"),
    (PY_CLASS, "_types.TypeDecorator"),
    (PY_CLASS, "abc.Collection"),
    (PY_CLASS, "advanced_alchemy.utils.dataclass.Empty"),
    (PY_CLASS, "jinja2.environment.Environment"),
    (PY_CLASS, "pydantic.BaseModel"),
    (PY_CLASS, "pydantic.generics.GenericModel"),
    (PY_CLASS, "pydantic.main.BaseModel"),
    (PY_CLASS, "redis.asyncio.Redis"),
    (PY_CLASS, "sqlalchemy.dialects.postgresql.named_types.ENUM"),
    (PY_CLASS, "sqlalchemy.orm.decl_api.DeclarativeMeta"),
    (PY_CLASS, "sqlalchemy.sql.sqltypes.TupleType"),
    (PY_CLASS, "valkey.asyncio.Valkey"),
    (PY_METH, "_types.TypeDecorator.process_bind_param"),
    (PY_METH, "_types.TypeDecorator.process_result_value"),
    (PY_METH, "litestar.typing.ParsedType.is_subclass_of"),
    (PY_METH, "type_engine"),
    # type vars and aliases / intentionally undocumented
    (PY_CLASS, "AnyIOBackend"),
    (PY_CLASS, "BaseSessionBackendT"),
    (PY_CLASS, "C"),
    (PY_CLASS, "CollectionT"),
    (PY_CLASS, "ControllerRouterHandler"),
    (PY_CLASS, "EmptyType"),
    (PY_CLASS, "ModelT"),
    (PY_CLASS, "PathParameterDefinition"),
    (PY_CLASS, "RouteHandlerType"),
    (PY_CLASS, "SelectT"),
    (PY_CLASS, "T"),
    (PY_OBJ, "litestar.security.base.AuthType"),
    # investigate
    (PY_CLASS, "Environment"),
    (PY_CLASS, "P"),
    (PY_CLASS, "pydantic_v1.BaseModel"),
    (PY_CLASS, "pydantic_v2.BaseModel"),
    (PY_CLASS, "advanced_alchemy.config.types.Empty"),
    (PY_OBJ, "litestar.template.base.TemplateType_co"),
    (PY_OBJ, "litestar.template.base.ContextType_co"),
    (PY_CLASS, "litestar.template.base.TemplateType_co"),
    (PY_CLASS, "litestar.template.base.ContextType_co"),
    (PY_CLASS, "litestar.template.base.R"),
    (PY_ATTR, "litestar.openapi.controller.OpenAPIController.swagger_ui_init_oauth"),
    # intentionally undocumented
    (PY_CLASS, "BacklogStrategy"),
    (PY_CLASS, "ExceptionT"),
    (PY_CLASS, "NoneType"),
    (PY_CLASS, "litestar._openapi.schema_generation.schema.SchemaCreator"),
    (PY_CLASS, "litestar._signature.model.SignatureModel"),
    (PY_CLASS, "litestar.contrib.sqlalchemy.plugins.init.config.compat._CreateEngineMixin"),
    (PY_CLASS, "litestar.utils.signature.ParsedSignature"),
    (PY_CLASS, "litestar.utils.sync.AsyncCallable"),
    # types in changelog that no longer exist
    (PY_ATTR, "litestar.dto.factory.DTOConfig.underscore_fields_private"),
    (PY_CLASS, "anyio.abc.BlockingPortal"),
    (PY_CLASS, "litestar.contrib.msgspec.MsgspecDTO"),
    (PY_CLASS, "litestar.contrib.repository.filters.NotInCollectionFilter"),
    (PY_CLASS, "litestar.contrib.repository.filters.NotInSearchFilter"),
    (PY_CLASS, "litestar.contrib.repository.filters.OnBeforeAfter"),
    (PY_CLASS, "litestar.contrib.repository.filters.OrderBy"),
    (PY_CLASS, "litestar.contrib.repository.filters.SearchFilter"),
    (PY_CLASS, "litestar.dto.base_factory.AbstractDTOFactory"),
    (PY_CLASS, "litestar.dto.factory.DTOConfig"),
    (PY_CLASS, "litestar.dto.factory.DTOData"),
    (PY_CLASS, "litestar.dto.interface.DTOInterface"),
    (PY_CLASS, "litestar.partial.Partial"),
    (PY_CLASS, "litestar.response.RedirectResponse"),
    (PY_CLASS, "litestar.response_containers.Redirect"),
    (PY_CLASS, "litestar.response_containers.Template"),
    (PY_CLASS, "litestar.contrib.sqlalchemy.plugins.SQLAlchemyPlugin"),
    (PY_CLASS, "litestar.contrib.sqlalchemy.plugins.SQLAlchemySerializationPlugin"),
    (PY_CLASS, "litestar.contrib.sqlalchemy.plugins.SQLAlchemyInitPlugin"),
    (PY_CLASS, "litestar.contrib.sqlalchemy.dto.SQLAlchemyDTO"),
    (PY_CLASS, "litestar.contrib.sqlalchemy.types.BigIntIdentity"),
    (PY_CLASS, "litestar.contrib.sqlalchemy.types.JsonB"),
    (PY_CLASS, "litestar.contrib.htmx.request.HTMXRequest"),
    (PY_CLASS, "litestar.typing.ParsedType"),
    (PY_METH, "litestar.dto.factory.DTOData.create_instance"),
    (PY_METH, "litestar.dto.interface.DTOInterface.data_to_encodable_type"),
    (PY_CLASS, "MetaData"),
    (PY_FUNC, "sqlalchemy.get_engine"),
    (PY_OBJ, "litestar.template.base.T_co"),
    ("py:exc", "RepositoryError"),
    ("py:exc", "InternalServerError"),
    ("py:exc", "HTTPExceptions"),
    (PY_CLASS, "litestar.template.Template"),
    (PY_CLASS, "litestar.middleware.compression.gzip_facade.GzipCompression"),
    (PY_CLASS, "litestar.handlers.http_handlers.decorators._subclass_warning"),
    (PY_CLASS, "litestar.background_tasks.P"),
    (PY_CLASS, "P.args"),
    (PY_CLASS, "P.kwargs"),
    (PY_CLASS, "litestar.contrib.jinja.P"),
    (PY_CLASS, "litestar.contrib.mako.P"),
    (PY_CLASS, "JWTDecodeOptions"),
    (PY_CLASS, "litestar.template.base.P"),
    (PY_CLASS, "litestar.contrib.pydantic.PydanticDTO"),
    (PY_CLASS, "litestar.contrib.pydantic.PydanticPlugin"),
    (PY_CLASS, "typing.Self"),
    (PY_CLASS, "attr.AttrsInstance"),
    (PY_CLASS, "typing_extensions.TypeGuard"),
    (PY_CLASS, "advanced_alchemy.types.BigIntIdentity"),
    (PY_CLASS, "advanced_alchemy.types.JsonB"),
    (PY_CLASS, "advanced_alchemy.repository.SQLAlchemyAsyncRepository"),
    # docs in flux as we prepare for `advanced_alchemy` 1.0 release. re-enable when finished
    (PY_CLASS, "advanced_alchemy.base.UUIDBase"),
    (PY_CLASS, "advanced_alchemy.base.UUIDAuditBase"),
    (PY_CLASS, "advanced_alchemy.base.BigIntBase"),
    (PY_CLASS, "advanced_alchemy.base.BigIntAuditBase"),
]

nitpick_ignore_regex = [
    (PY_ATTR, "litestar.repository.testing.AsyncGenericMockRepository.id_attribute"),
    (PY_ATTR, "litestar.repository.AbstractAsyncRepository.id_attribute"),
    (PY_ATTR, "litestar.repository.AbstractSyncRepository.id_attribute"),
    # (PY_ATTR, "litestar.repository.AsyncGenericMockRepository.id_attribute"),
    (PY_OBJ, r"typing\..*"),
    (PY_RE, r".*R_co"),
    (PY_RE, r".*UserType"),
    (PY_RE, r"ModelT"),
    (PY_RE, r"litestar.*\.T"),
    (PY_RE, r"litestar.contrib.sqlalchemy.repository.ModelT"),
    (PY_RE, r"litestar\.middleware\.session\.base\.BaseSessionBackendT"),
    (PY_RE, r"litestar\.types.*"),
    (PY_RE, r"httpx.*"),
    # type vars
    (PY_RE, r"litestar.middleware.session.base.ConfigT"),
    (PY_RE, r"litestar\.connection\.base\.AuthT"),
    (PY_RE, r"litestar\.connection\.base\.HandlerT"),
    (PY_RE, r"litestar\.connection\.base\.StateT"),
    (PY_RE, r"litestar\.connection\.base\.UserT"),
    (PY_RE, r"litestar\.pagination\.C"),
    (PY_RE, r"multidict\..*"),
    (PY_RE, r"advanced_alchemy.*\.T"),
    (PY_RE, r"advanced_alchemy\.config.common\.EngineT"),
    (PY_RE, r"advanced_alchemy\.config.common\.SessionT"),
    (PY_RE, r".*R"),
    (PY_OBJ, r"litestar.security.jwt.auth.TokenT"),
    (PY_CLASS, "ExceptionToProblemDetailMapType"),
    (PY_CLASS, "litestar.security.jwt.token.JWTDecodeOptions"),
]

# Warnings about missing references to those targets in the specified location will be ignored.
# The source of the references is taken 1:1 from the warnings as reported by Sphinx, e.g
# **/litestar/testing/client/async_client.py:docstring of litestar.testing.AsyncTestClient.exit_stack:1: WARNING: py:class reference target not found: AsyncExitStack
# would be added as: "litestar.testing.AsyncTestClient.exit_stack": {"AsyncExitStack"},
ignore_missing_refs = {
    # No idea what autodoc is doing here. Possibly unfixable on our end
    "litestar.template.base.TemplateEngineProtocol.get_template": {"litestar.template.base.T_co"},
    "litestar.template": {"litestar.template.base.T_co"},
    "litestar.openapi.OpenAPIController.security": {"SecurityRequirement"},
    "litestar.response.file.async_file_iterator": {"FileSystemAdapter"},
    re.compile("litestar.response.redirect.*"): {"RedirectStatusType"},
    re.compile(r"litestar\.plugins.*"): re.compile(".*ModelT"),
    re.compile(r"litestar\.(contrib|repository)\.*"): re.compile(".*T"),
    re.compile(r"litestar\.contrib\.sqlalchemy\.*"): re.compile(
        ".*(ConnectionT|EngineT|SessionT|SessionMakerT|SlotsBase)"
    ),
    re.compile(r"litestar\.dto.*"): re.compile(".*T|.*FieldDefinition|Empty"),
    re.compile(r"litestar\.template\.(config|TemplateConfig).*"): re.compile(".*EngineType"),
    "litestar.concurrency.set_asyncio_executor": {"ThreadPoolExecutor"},
    "litestar.concurrency.get_asyncio_executor": {"ThreadPoolExecutor"},
    re.compile(r"litestar\.channels\.backends\.asyncpg.*"): {"asyncpg.connection.Connection", "asyncpg.Connection"},
    re.compile(r"litestar\.handlers\.websocket_handlers\.stream.*"): {"WebSocketMode"},
}

# Do not warn about broken links to the following:
linkcheck_ignore = [
    r"http://localhost(:\d+)?",
    r"http://127.0.0.1(:\d+)?",
    "http://testserver",
]

auto_pytabs_min_version = (3, 8)
auto_pytabs_max_version = (3, 11)
auto_pytabs_compat_mode = True

autosectionlabel_prefix_document = True

suppress_warnings = [
    "autosectionlabel.*",
    "ref.python",  # TODO: remove when https://github.com/sphinx-doc/sphinx/issues/4961 is fixed
]

html_theme = "litestar_sphinx_theme"
html_static_path = ["_static"]
html_js_files = ["versioning.js"]
html_css_files = ["style.css"]
html_show_sourcelink = False
html_title = "Litestar Framework"

html_theme_options = {
    "use_page_nav": False,
    "github_repo_name": "litestar",
    "logo": {
        "link": "https://litestar.dev",
    },
    "pygment_light_style": "xcode",
    "pygment_dark_style": "lightbulb",
    "navigation_with_keys": True,
    "extra_navbar_items": {
        "Documentation": "index",
        "Community": {
            "Contributing": {
                "description": "Learn how to contribute to the Litestar project",
                "link": "https://docs.litestar.dev/latest/contribution-guide.html",
                "icon": "contributing",
            },
            "Code of Conduct": {
                "description": "Review the etiquette for interacting with the Litestar community",
                "link": "https://github.com/litestar-org/.github?tab=coc-ov-file",
                "icon": "coc",
            },
            "Security": {
                "description": "Overview of Litestar's security protocols",
                "link": "https://github.com/litestar-org/.github?tab=coc-ov-file#security-ov-file",
                "icon": "coc",
            },
        },
        "About": {
            "Litestar Organization": {
                "description": "Details about the Litestar organization",
                "link": "https://litestar.dev/about/organization",
                "icon": "org",
            },
            "Releases": {
                "description": "Explore the release process, versioning, and deprecation policy for Litestar",
                "link": "https://litestar.dev/about/litestar-releases",
                "icon": "releases",
            },
        },
        "Release notes": {
            "What's new in 2.0": "release-notes/whats-new-2",
            "2.x Changelog": "https://docs.litestar.dev/2/release-notes/changelog.html",
            "1.x Changelog": "https://docs.litestar.dev/1/release-notes/changelog.html",
        },
        "Help": {
            "Discord Help Forum": {
                "description": "Dedicated Discord help forum",
                "link": "https://discord.gg/litestar",
                "icon": "coc",
            },
            "GitHub Discussions": {
                "description": "GitHub Discussions ",
                "link": "https://github.com/orgs/litestar-org/discussions",
                "icon": "coc",
            },
            "Stack Overflow": {
                "description": "We monitor the <code><b>litestar</b></code> tag on Stack Overflow",
                "link": "https://stackoverflow.com/questions/tagged/litestar",
                "icon": "coc",
            },
        },
    },
}


def update_html_context(
    app: Sphinx, pagename: str, templatename: str, context: dict[str, Any], doctree: document
) -> None:
    context["generate_toctree_html"] = partial(context["generate_toctree_html"], startdepth=0)


def delayed_setup(app: Sphinx) -> None:
    """
    When running linkcheck pydata_sphinx_theme causes a build failure, and checking
    the builder in the initial `setup` function call is not possible, so the check
    and extension setup has to be delayed until the builder is initialized.

    When running gettext pydata_sphinx_theme causes a build failure by attempting to
    write to the _static directory, resulting in a FileNotFoundError for missing files
    like pygments.css. See: https://github.com/pydata/pydata-sphinx-theme/issues/1840
    """
    if app.builder.name == "linkcheck" or app.builder.name == "gettext":
        return

    app.setup_extension("pydata_sphinx_theme")
    app.connect("html-page-context", update_html_context)  # type: ignore


def setup(app: Sphinx) -> dict[str, bool]:
    app.connect("builder-inited", delayed_setup, priority=0)  # type: ignore

    app.setup_extension("litestar_sphinx_theme")

    return {"parallel_read_safe": True, "parallel_write_safe": True}



from datetime import datetime, timedelta

from litestar import Litestar, Request
from litestar.stores.memory import MemoryStore

memory_store = MemoryStore()


async def after_response(request: Request) -> None:
    now = datetime.utcnow()
    last_cleared = request.app.state.get("store_last_cleared", now)
    if datetime.utcnow() - last_cleared > timedelta(seconds=30):
        await memory_store.delete_expired()
    app.state["store_last_cleared"] = now


app = Litestar(after_response=after_response)



from __future__ import annotations

from typing import Any

from litestar.exceptions import responses
from litestar.utils.deprecation import warn_deprecation


def __getattr__(name: str) -> Any:
    if name == "create_debug_response":
        warn_deprecation(
            version="2.9",
            deprecated_name=name,
            kind="function",
            removal_in="3.0",
            alternative="litestar.exceptions.responses.create_debug_response",
        )
        return responses.create_debug_response

    raise AttributeError(f"module {__name__} has no attribute {name}")



from litestar import Litestar
from litestar.static_files.config import StaticFilesConfig

app = Litestar(
    static_files_config=[
        StaticFilesConfig(directories=["assets"], path="/static"),
    ],
)



from __future__ import annotations

import contextlib
import importlib
import inspect
import os
import re
import sys
from dataclasses import dataclass
from datetime import datetime, timedelta, timezone
from functools import wraps
from importlib.util import find_spec
from itertools import chain
from os import getenv
from pathlib import Path
from typing import TYPE_CHECKING, Any, Callable, Generator, Iterable, Sequence, TypeVar, cast

from litestar.cli._suggestions import suggest_option

try:
    from rich_click import NoSuchOption
    from rich_click import RichCommand as Command
    from rich_click import RichGroup as Group
except ImportError:
    from click import Command, Group, NoSuchOption  # type: ignore[assignment]

from click import ClickException, Context, pass_context
from rich import get_console
from rich.table import Table
from typing_extensions import ParamSpec, get_type_hints

from litestar import Litestar, __version__
from litestar.middleware import DefineMiddleware
from litestar.utils import get_name

if sys.version_info >= (3, 10):
    from importlib.metadata import entry_points
else:
    from importlib_metadata import entry_points


if TYPE_CHECKING:
    from types import ModuleType

    try:
        from rich_click import RichContext
    except ImportError:
        RichContext = Context  # type: ignore[assignment,misc]

    from litestar.openapi import OpenAPIConfig
    from litestar.routes import ASGIRoute, HTTPRoute, WebSocketRoute
    from litestar.types import AnyCallable


UVICORN_INSTALLED = find_spec("uvicorn") is not None
JSBEAUTIFIER_INSTALLED = find_spec("jsbeautifier") is not None


__all__ = (
    "JSBEAUTIFIER_INSTALLED",
    "UVICORN_INSTALLED",
    "LitestarCLIException",
    "LitestarEnv",
    "LitestarExtensionGroup",
    "LitestarGroup",
    "LoadedApp",
    "show_app_info",
)


P = ParamSpec("P")
T = TypeVar("T")


AUTODISCOVERY_FILE_NAMES = ["app", "application"]

console = get_console()


class LitestarCLIException(ClickException):
    """Base class for Litestar CLI exceptions."""

    def __init__(self, message: str) -> None:
        """Initialize exception and style error message."""
        super().__init__(message)


@dataclass
class LitestarEnv:
    """Information about the current Litestar environment variables."""

    app_path: str
    app: Litestar
    cwd: Path
    host: str | None = None
    port: int | None = None
    is_app_factory: bool = False

    @classmethod
    def from_env(cls, app_path: str | None, app_dir: Path | None = None) -> LitestarEnv:
        """Load environment variables.

        If ``python-dotenv`` is installed, use it to populate environment first
        """
        cwd = Path().cwd() if app_dir is None else app_dir
        cwd_str_path = str(cwd)
        if cwd_str_path not in sys.path:
            sys.path.append(cwd_str_path)

        with contextlib.suppress(ImportError):
            import dotenv

            dotenv.load_dotenv()
        app_path = app_path or getenv("LITESTAR_APP")
        app_name = getenv("LITESTAR_APP_NAME") or "Litestar"
        quiet_console = getenv("LITESTAR_QUIET_CONSOLE") or False
        if app_path and getenv("LITESTAR_APP") is None:
            os.environ["LITESTAR_APP"] = app_path
        if app_path:
            if not quiet_console and isatty():
                console.print(f"Using {app_name} app from env: [bright_blue]{app_path!r}")
            loaded_app = _load_app_from_path(app_path)
        else:
            loaded_app = _autodiscover_app(cwd)

        port = getenv("LITESTAR_PORT")

        return cls(
            app_path=loaded_app.app_path,
            app=loaded_app.app,
            host=getenv("LITESTAR_HOST"),
            port=int(port) if port else None,
            is_app_factory=loaded_app.is_factory,
            cwd=cwd,
        )


@dataclass
class LoadedApp:
    """Information about a loaded Litestar app."""

    app: Litestar
    app_path: str
    is_factory: bool


class LitestarGroup(Group):  # pyright: ignore
    """:class:`click.Group` subclass that automatically injects ``app`` and ``env` kwargs into commands that request it.

    Use this as the ``cls`` for :class:`click.Group` if you're extending the internal CLI with a group. For ``command``s
    added directly to the root group this is not needed.
    """

    def __init__(
        self,
        name: str | None = None,
        commands: dict[str, Command] | Sequence[Command] | None = None,
        **attrs: Any,
    ) -> None:
        """Init ``LitestarGroup``"""
        self.group_class = LitestarGroup
        super().__init__(name=name, commands=commands, **attrs)  # type: ignore[arg-type]

    def add_command(self, cmd: Command, name: str | None = None) -> None:  # type: ignore[override]
        """Add command.

        If necessary, inject ``app`` and ``env`` kwargs
        """
        if cmd.callback:
            cmd.callback = _inject_args(cmd.callback)
        super().add_command(cmd)

    def command(self, *args: Any, **kwargs: Any) -> Callable[[AnyCallable], Command] | Command:  # type: ignore[override]
        # For some reason, even when copying the overloads + signature from click 1:1, mypy goes haywire
        """Add a function as a command.

        If necessary, inject ``app`` and ``env`` kwargs
        """

        def decorator(f: AnyCallable) -> Command:
            f = _inject_args(f)
            return cast("Command", Group.command(self, *args, **kwargs)(f))

        return decorator

    def parse_args(self, *args: Any, **kwargs: Any) -> list[str]:
        try:
            return super().parse_args(*args, **kwargs)
        except NoSuchOption as e:
            suggest_option(e)


class LitestarExtensionGroup(LitestarGroup):
    """``LitestarGroup`` subclass that will load Litestar-CLI extensions from the `litestar.commands` entry_point.

    This group class should not be used on any group besides the root ``litestar_group``.
    """

    def __init__(
        self,
        name: str | None = None,
        commands: dict[str, Command] | Sequence[Command] | None = None,
        **attrs: Any,
    ) -> None:
        """Init ``LitestarExtensionGroup``"""
        super().__init__(name=name, commands=commands, **attrs)

        self._prepare_done = False

        for entry_point in entry_points(group="litestar.commands"):
            command = entry_point.load()
            _wrap_commands([command])
            self.add_command(command, entry_point.name)

    def _prepare(self, ctx: Context) -> None:
        if self._prepare_done:
            return

        if isinstance(ctx.obj, LitestarEnv):
            env: LitestarEnv | None = ctx.obj
        else:
            try:
                app_path = ctx.params.get("app_path")
                app_dir = ctx.params.get("app_dir")
                env = ctx.obj = LitestarEnv.from_env(app_path, app_dir)
            except LitestarCLIException:
                env = None

        if env:
            for plugin in env.app.plugins.cli:
                plugin.on_cli_init(self)

        self._prepare_done = True

    def make_context(
        self,
        info_name: str | None,
        args: list[str],
        parent: Context | None = None,
        **extra: Any,
    ) -> Context:
        ctx = super().make_context(info_name, args, parent, **extra)
        self._prepare(ctx)
        return ctx

    def list_commands(self, ctx: Context) -> list[str]:
        self._prepare(ctx)
        return super().list_commands(ctx)

    def format_help(self, ctx: Context, formatter: Any) -> None:
        """Override format_help to ensure plugins are loaded before rendering help."""
        self._prepare(ctx)
        return super().format_help(ctx, formatter)  # type: ignore[arg-type]


def _inject_args(func: Callable[P, T]) -> Callable[P, T]:
    """Inject the app instance into a ``Command``"""
    params = inspect.signature(func).parameters

    @wraps(func)
    def wrapped(ctx: Context, /, *args: P.args, **kwargs: P.kwargs) -> T:
        needs_app = "app" in params
        needs_env = "env" in params
        if needs_env or needs_app:
            # only resolve this if actually requested. Commands that don't need an env or app should be able to run
            # without
            if not isinstance(ctx.obj, LitestarEnv):
                ctx.obj = ctx.obj()
            env = ctx.ensure_object(LitestarEnv)
            if needs_app:
                kwargs["app"] = env.app
            if needs_env:
                kwargs["env"] = env

        if "ctx" in params:
            kwargs["ctx"] = ctx

        return func(*args, **kwargs)

    return pass_context(wrapped)


def _wrap_commands(commands: Iterable[Command]) -> None:
    for command in commands:
        if hasattr(command, "commands"):
            _wrap_commands(command.commands.values())  # pyright: ignore[reportGeneralTypeIssues]
        elif command.callback:
            command.callback = _inject_args(command.callback)


def _bool_from_env(key: str, default: bool = False) -> bool:
    value = getenv(key)
    if not value:
        return default
    value = value.lower()
    return value in ("true", "1")


def _validate_app_path(app_path: str) -> tuple[ModuleType, str]:
    try:
        module_path, app_name = app_path.split(":")
    except ValueError:
        console.print(f"[bold red] Invalid argument passed --app {app_path!r}: Expected 'module_path:app'")
        sys.exit(1)

    try:
        module = importlib.import_module(module_path)
    except ImportError as e:
        if e.name == module_path:
            console.print(f"[bold red] Invalid argument passed --app {app_path!r}: Module not found")
            sys.exit(1)
        else:
            raise (e)
    return module, app_name


def _load_app_from_path(app_path: str) -> LoadedApp:
    module, app_name = _validate_app_path(app_path)
    app = getattr(module, app_name)
    is_factory = False
    if not isinstance(app, Litestar) and callable(app):
        app = app()
        is_factory = True
    return LoadedApp(app=app, app_path=app_path, is_factory=is_factory)


def _path_to_dotted_path(path: Path) -> str:
    if path.stem == "__init__":
        path = path.parent
    return ".".join(path.with_suffix("").parts)


def _arbitrary_autodiscovery_paths(base_dir: Path) -> Generator[Path, None, None]:
    yield from _autodiscovery_paths(base_dir, arbitrary=False)
    for path in base_dir.iterdir():
        if path.name.startswith(".") or path.name.startswith("_"):
            continue
        if path.is_file() and path.suffix == ".py":
            yield path


def _autodiscovery_paths(base_dir: Path, arbitrary: bool = True) -> Generator[Path, None, None]:
    for name in AUTODISCOVERY_FILE_NAMES:
        path = base_dir / name

        if path.exists() or path.with_suffix(".py").exists():
            yield path
        if arbitrary and path.is_dir():
            yield from _arbitrary_autodiscovery_paths(path)


def _autodiscover_app(cwd: Path) -> LoadedApp:
    app_name = getenv("LITESTAR_APP_NAME") or "Litestar"
    quiet_console = getenv("LITESTAR_QUIET_CONSOLE") or False
    for file_path in _autodiscovery_paths(cwd):
        import_path = _path_to_dotted_path(file_path.relative_to(cwd))
        module = importlib.import_module(import_path)

        for attr, value in chain(
            [("app", getattr(module, "app", None)), ("application", getattr(module, "application", None))],
            module.__dict__.items(),
        ):
            if isinstance(value, Litestar):
                app_string = f"{import_path}:{attr}"
                os.environ["LITESTAR_APP"] = app_string
                if not quiet_console and isatty():
                    console.print(f"Using {app_name} app from [bright_blue]{app_string}")
                return LoadedApp(app=value, app_path=app_string, is_factory=False)

        if hasattr(module, "create_app"):
            app_string = f"{import_path}:create_app"
            os.environ["LITESTAR_APP"] = app_string
            if not quiet_console and isatty():
                console.print(f"Using {app_name} factory from [bright_blue]{app_string}")
            return LoadedApp(app=module.create_app(), app_path=app_string, is_factory=True)

        for attr, value in module.__dict__.items():
            if not callable(value):
                continue
            return_annotation = (
                get_type_hints(value, include_extras=True).get("return") if hasattr(value, "__annotations__") else None
            )
            if not return_annotation:
                continue
            if return_annotation in ("Litestar", Litestar):
                app_string = f"{import_path}:{attr}"
                os.environ["LITESTAR_APP"] = app_string
                if not quiet_console and sys.stdout.isatty():
                    console.print(f"Using {app_name} factory from [bright_blue]{app_string}")
                return LoadedApp(app=value(), app_path=f"{app_string}", is_factory=True)

    raise LitestarCLIException(f"Could not find {app_name} instance or factory")


def _format_is_enabled(value: Any) -> str:
    """Return a coloured string `"Enabled" if ``value`` is truthy, else "Disabled"."""
    return "[green]Enabled[/]" if value else "[red]Disabled[/]"


def show_app_info(app: Litestar) -> None:  # pragma: no cover
    """Display basic information about the application and its configuration."""

    table = Table(show_header=False)
    table.add_column("title", style="cyan")
    table.add_column("value", style="bright_blue")

    table.add_row("Litestar version", f"{__version__.major}.{__version__.minor}.{__version__.patch}")
    table.add_row("Debug mode", _format_is_enabled(app.debug))
    table.add_row("Python Debugger on exception", _format_is_enabled(app.pdb_on_exception))
    table.add_row("CORS", _format_is_enabled(app.cors_config))
    table.add_row("CSRF", _format_is_enabled(app.csrf_config))
    if app.allowed_hosts:
        allowed_hosts = app.allowed_hosts

        table.add_row("Allowed hosts", ", ".join(allowed_hosts.allowed_hosts))

    openapi_enabled = _format_is_enabled(app.openapi_config)
    if app.openapi_config:
        path = (
            app.openapi_config.openapi_controller.path
            if app.openapi_config.openapi_controller
            else app.openapi_config.path or "/schema"
        )
        openapi_enabled += f" path=[yellow]{path}"
    table.add_row("OpenAPI", openapi_enabled)

    table.add_row("Compression", app.compression_config.backend if app.compression_config else "[red]Disabled")

    if app.template_engine:
        table.add_row("Template engine", type(app.template_engine).__name__)

    if app.static_files_config:
        static_files_configs = app.static_files_config
        static_files_info = [
            f"path=[yellow]{static_files.path}[/] dirs=[yellow]{', '.join(map(str, static_files.directories))}[/] "
            f"html_mode={_format_is_enabled(static_files.html_mode)}"
            for static_files in static_files_configs
        ]
        table.add_row("Static files", "\n".join(static_files_info))

    middlewares = []
    for middleware in app.middleware:
        updated_middleware = middleware.middleware if isinstance(middleware, DefineMiddleware) else middleware
        middlewares.append(get_name(updated_middleware))
    if middlewares:
        table.add_row("Middlewares", ", ".join(middlewares))

    console.print(table)


def validate_ssl_file_paths(certfile_arg: str | None, keyfile_arg: str | None) -> tuple[str, str] | tuple[None, None]:
    """Validate whether given paths exist, are not directories and were both provided or none was. Return the resolved paths.

    Args:
        certfile_arg: path argument for the certificate file
        keyfile_arg: path argument for the key file

    Returns:
        tuple of resolved paths converted to str or tuple of None's if no argument was provided
    """
    if certfile_arg is None and keyfile_arg is None:
        return (None, None)

    resolved_paths = []

    for argname, arg in {"--ssl-certfile": certfile_arg, "--ssl-keyfile": keyfile_arg}.items():
        if arg is None:
            raise LitestarCLIException(f"No value provided for {argname}")
        path = Path(arg).resolve()
        if path.is_dir():
            raise LitestarCLIException(f"Path provided for {argname} is a directory: {path}")
        if not path.exists():
            raise LitestarCLIException(f"File provided for {argname} was not found: {path}")
        resolved_paths.append(str(path))

    return tuple(resolved_paths)  # type: ignore[return-value]


def create_ssl_files(
    certfile_arg: str | None, keyfile_arg: str | None, common_name: str = "localhost"
) -> tuple[str, str]:
    """Validate whether both files were provided, are not directories, their parent dirs exist and either both files exists or none does.
    If neither file exists, create a self-signed ssl certificate and a passwordless key at the location.

    Args:
        certfile_arg: path argument for the certificate file
        keyfile_arg: path argument for the key file
        common_name: the CN to be used as cert issuer and subject

    Returns:
        resolved paths of the found or generated files
    """
    resolved_paths = []

    for argname, arg in {"--ssl-certfile": certfile_arg, "--ssl-keyfile": keyfile_arg}.items():
        if arg is None:
            raise LitestarCLIException(f"No value provided for {argname}")
        path = Path(arg).resolve()
        if path.is_dir():
            raise LitestarCLIException(f"Path provided for {argname} is a directory: {path}")
        if not (parent_dir := path.parent).exists():
            raise LitestarCLIException(
                f"Could not create file, parent directory for {argname} doesn't exist: {parent_dir}"
            )
        resolved_paths.append(path)

    if (not resolved_paths[0].exists()) ^ (not resolved_paths[1].exists()):
        raise LitestarCLIException(
            "Both certificate and key file must exists or both must not exists when using --create-self-signed-cert"
        )

    if (not resolved_paths[0].exists()) and (not resolved_paths[1].exists()):
        _generate_self_signed_cert(resolved_paths[0], resolved_paths[1], common_name)

    return (str(resolved_paths[0]), str(resolved_paths[1]))


def _generate_self_signed_cert(certfile_path: Path, keyfile_path: Path, common_name: str) -> None:
    """Create a self-signed certificate using the cryptography modules at given paths"""
    try:
        from cryptography import x509
        from cryptography.hazmat.backends import default_backend
        from cryptography.hazmat.primitives import hashes, serialization
        from cryptography.hazmat.primitives.asymmetric import rsa
        from cryptography.x509.oid import NameOID
    except ImportError as err:
        raise LitestarCLIException(
            "Cryptography must be installed when using --create-self-signed-cert\nPlease install the litestar[cryptography] extras"
        ) from err

    subject = x509.Name(
        [
            x509.NameAttribute(NameOID.COMMON_NAME, common_name),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, "Development Certificate"),
        ]
    )

    key = rsa.generate_private_key(public_exponent=65537, key_size=2048, backend=default_backend())

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(subject)
        .public_key(key.public_key())
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.now(tz=timezone.utc))
        .not_valid_after(datetime.now(tz=timezone.utc) + timedelta(days=365))
        .add_extension(x509.SubjectAlternativeName([x509.DNSName(common_name)]), critical=False)
        .add_extension(x509.ExtendedKeyUsage([x509.OID_SERVER_AUTH]), critical=False)
        .sign(key, hashes.SHA256(), default_backend())
    )

    with certfile_path.open("wb") as cert_file:
        cert_file.write(cert.public_bytes(serialization.Encoding.PEM))

    with keyfile_path.open("wb") as key_file:
        key_file.write(
            key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.NoEncryption(),
            )
        )


def remove_routes_with_patterns(
    routes: list[HTTPRoute | ASGIRoute | WebSocketRoute], patterns: tuple[str, ...]
) -> list[HTTPRoute | ASGIRoute | WebSocketRoute]:
    regex_routes = []
    valid_patterns = []
    for pattern in patterns:
        try:
            check_pattern = re.compile(pattern)
            valid_patterns.append(check_pattern)
        except re.error as e:
            console.print(f"Error: {e}. Invalid regex pattern supplied: '{pattern}'. Omitting from querying results.")

    for route in routes:
        checked_pattern_route_matches = []
        for pattern_compile in valid_patterns:
            matches = pattern_compile.match(route.path)
            checked_pattern_route_matches.append(matches)

        if not any(checked_pattern_route_matches):
            regex_routes.append(route)

    return regex_routes


def remove_default_schema_routes(
    routes: list[HTTPRoute | ASGIRoute | WebSocketRoute], openapi_config: OpenAPIConfig
) -> list[HTTPRoute | ASGIRoute | WebSocketRoute]:
    schema_path = (
        (openapi_config.path or "/schema")
        if openapi_config.openapi_controller is None
        else openapi_config.openapi_controller.path
    )
    return remove_routes_with_patterns(routes, (schema_path,))


def isatty() -> bool:
    """Detect if a terminal is TTY enabled.

    This is a convenience wrapper around the built in system methods.  This allows for easier testing of TTY/non-TTY modes.
    """
    return sys.stdout.isatty()



from datetime import datetime

from sqlalchemy.orm import Mapped, mapped_column
from typing_extensions import Annotated

from litestar import Litestar, post
from litestar.dto import DTOConfig, dto_field
from litestar.plugins.sqlalchemy import SQLAlchemyDTO

from .my_lib import Base


class User(Base):
    name: Mapped[str]
    password: Mapped[str] = mapped_column(info=dto_field("private"))
    created_at: Mapped[datetime] = mapped_column(info=dto_field("read-only"))


config = DTOConfig(rename_fields={"name": "userName"})
UserDTO = SQLAlchemyDTO[Annotated[User, config]]


@post("/users", dto=UserDTO, sync_to_thread=False)
def create_user(data: User) -> User:
    assert data.name == "Litestar User"
    data.created_at = datetime.min
    return data


app = Litestar(route_handlers=[create_user])

# run: /users -H "Content-Type: application/json" -d '{"userName":"Litestar User","password":"xyz","created_at":"2023-04-24T00:00:00Z"}'



from typing import Any, Dict, List, Union

from litestar import Litestar, post

TODO_LIST: List[Dict[str, Union[str, bool]]] = []


@post("/")
async def add_item(data: Dict[str, Any]) -> List[Dict[str, Union[str, bool]]]:
    TODO_LIST.append(data)
    return TODO_LIST


app = Litestar([add_item])



from __future__ import annotations

from dataclasses import dataclass, field
from uuid import UUID, uuid4

from litestar import Litestar, post
from litestar.dto import DataclassDTO, DTOConfig


@dataclass
class User:
    name: str
    email: str
    age: int
    id: UUID = field(default_factory=uuid4)


class UserWriteDTO(DataclassDTO[User]):
    """Don't allow client to set the id."""

    config = DTOConfig(exclude={"id"})


# We need a dto for the handler to parse the request data per the configuration, however,
# we don't need a return DTO as we are returning a dataclass, and Litestar already knows
# how to serialize dataclasses.
@post("/users", dto=UserWriteDTO, return_dto=None, sync_to_thread=False)
def create_user(data: User) -> User:
    """Create an user."""
    return data


app = Litestar(route_handlers=[create_user])

# run: /users -H "Content-Type: application/json" -d '{"name":"Peter","email": "peter@example.com", "age":41}'



import json
from typing import TYPE_CHECKING

from litestar import Litestar, asgi
from litestar.response.base import ASGIResponse

if TYPE_CHECKING:
    from litestar.types import Receive, Scope, Send


@asgi("/some/sub-path", is_mount=True, copy_scope=True)
async def my_asgi_app(scope: "Scope", receive: "Receive", send: "Send") -> None:
    """
    Args:
        scope: The ASGI connection scope.
        receive: The ASGI receive function.
        send: The ASGI send function.

    Returns:
        None
    """
    body = json.dumps({"forwarded_path": scope["path"]})
    response = ASGIResponse(body=body.encode("utf-8"))
    await response(scope, receive, send)


app = Litestar(route_handlers=[my_asgi_app])



import pytest
from advanced_alchemy.extensions import litestar as sa_litestar
from advanced_alchemy.extensions.litestar import base as sa_base
from advanced_alchemy.extensions.litestar import exceptions as sa_exceptions
from advanced_alchemy.extensions.litestar import filters as sa_filters
from advanced_alchemy.extensions.litestar import mixins as sa_mixins
from advanced_alchemy.extensions.litestar import repository as sa_repository
from advanced_alchemy.extensions.litestar import service as sa_service
from advanced_alchemy.extensions.litestar import types as sa_types
from advanced_alchemy.extensions.litestar import utils as sa_utils

from litestar.pagination import OffsetPagination
from litestar.plugins import sqlalchemy


def test_re_exports() -> None:
    assert sqlalchemy.base is sa_base
    assert sqlalchemy.filters is sa_filters
    assert sqlalchemy.types is sa_types
    assert sqlalchemy.mixins is sa_mixins
    assert sqlalchemy.utils is sa_utils
    assert sqlalchemy.repository is sa_repository
    assert sqlalchemy.service is sa_service
    assert sqlalchemy.exceptions is sa_exceptions
    assert OffsetPagination is sa_service.OffsetPagination

    assert sqlalchemy.AlembicAsyncConfig is sa_litestar.AlembicAsyncConfig
    assert sqlalchemy.AlembicCommands is sa_litestar.AlembicCommands
    assert sqlalchemy.AlembicSyncConfig is sa_litestar.AlembicSyncConfig
    assert sqlalchemy.AsyncSessionConfig is sa_litestar.AsyncSessionConfig
    assert sqlalchemy.EngineConfig is sa_litestar.EngineConfig
    assert sqlalchemy.SQLAlchemyAsyncConfig is sa_litestar.SQLAlchemyAsyncConfig
    assert sqlalchemy.SQLAlchemyDTO is sa_litestar.SQLAlchemyDTO
    assert sqlalchemy.SQLAlchemyDTOConfig is sa_litestar.SQLAlchemyDTOConfig
    assert sqlalchemy.SQLAlchemyInitPlugin is sa_litestar.SQLAlchemyInitPlugin
    assert sqlalchemy.SQLAlchemyPlugin is sa_litestar.SQLAlchemyPlugin
    assert sqlalchemy.SQLAlchemySerializationPlugin is sa_litestar.SQLAlchemySerializationPlugin
    assert sqlalchemy.SQLAlchemySyncConfig is sa_litestar.SQLAlchemySyncConfig
    assert sqlalchemy.SyncSessionConfig is sa_litestar.SyncSessionConfig

    # deprecated, to be removed later
    with pytest.warns(DeprecationWarning):
        assert sqlalchemy.AuditColumns is sa_base.AuditColumns
        assert sqlalchemy.BigIntAuditBase is sa_base.BigIntAuditBase
        assert sqlalchemy.BigIntBase is sa_base.BigIntBase
        assert sqlalchemy.BigIntPrimaryKey is sa_base.BigIntPrimaryKey
        assert sqlalchemy.CommonTableAttributes is sa_base.CommonTableAttributes
        assert sqlalchemy.UUIDAuditBase is sa_base.UUIDAuditBase
        assert sqlalchemy.UUIDBase is sa_base.UUIDBase
        assert sqlalchemy.UUIDPrimaryKey is sa_base.UUIDPrimaryKey
        assert sqlalchemy.orm_registry is sa_base.orm_registry



from litestar.plugins.sqlalchemy import SQLAlchemyAsyncConfig, SQLAlchemyPlugin

sqlalchemy_config = SQLAlchemyAsyncConfig(connection_string="sqlite+aiosqlite:///test.sqlite")
plugin = SQLAlchemyPlugin(config=sqlalchemy_config)



from typing import List

from piccolo.columns import Boolean, Varchar
from piccolo.table import Table, create_db_tables

from litestar import Litestar, MediaType, delete, get, patch, post
from litestar.contrib.piccolo import PiccoloDTO
from litestar.dto import DTOConfig, DTOData
from litestar.exceptions import NotFoundException

from .piccolo_conf import DB


class Task(Table, db=DB):
    """
    An example table.
    """

    name = Varchar()
    completed = Boolean(default=False)


class PatchDTO(PiccoloDTO[Task]):
    """Allow partial updates."""

    config = DTOConfig(exclude={"id"}, partial=True)


@get(
    "/tasks",
    return_dto=PiccoloDTO[Task],
    media_type=MediaType.JSON,
    tags=["Task"],
)
async def tasks() -> List[Task]:
    return await Task.select().order_by(Task.id, ascending=False)


@post(
    "/tasks",
    dto=PiccoloDTO[Task],
    return_dto=PiccoloDTO[Task],
    media_type=MediaType.JSON,
    tags=["Task"],
)
async def create_task(data: Task) -> Task:
    await data.save()
    await data.refresh()
    return data


@patch(
    "/tasks/{task_id:int}",
    dto=PatchDTO,
    return_dto=PiccoloDTO[Task],
    media_type=MediaType.JSON,
    tags=["Task"],
)
async def update_task(task_id: int, data: DTOData[Task]) -> Task:
    task = await Task.objects().get(Task.id == task_id)
    if not task:
        raise NotFoundException("Task does not exist")
    result = data.update_instance(task)
    await result.save()
    return result


@delete("/tasks/{task_id:int}", tags=["Task"])
async def delete_task(task_id: int) -> None:
    task = await Task.objects().get(Task.id == task_id)
    if not task:
        raise NotFoundException("Task does not exist")
    await task.remove()


async def on_startup():
    await create_db_tables(Task, if_not_exists=True)


app = Litestar(route_handlers=[tasks, create_task, delete_task, update_task], on_startup=[on_startup], debug=True)



from typing import Dict

from litestar import Litestar, get
from litestar.openapi.config import OpenAPIConfig
from litestar.openapi.plugins import ScalarRenderPlugin


@get("/", sync_to_thread=False)
def hello_world() -> Dict[str, str]:
    return {"message": "Hello World"}


app = Litestar(
    route_handlers=[hello_world],
    openapi_config=OpenAPIConfig(
        title="Litestar Example",
        description="Example of Litestar with Scalar OpenAPI docs",
        version="0.0.1",
        render_plugins=[ScalarRenderPlugin()],
    ),
)



from __future__ import annotations

from typing import TYPE_CHECKING

from sqlalchemy import text

from litestar import Litestar, get
from litestar.plugins.sqlalchemy import SQLAlchemyInitPlugin, SQLAlchemySyncConfig

if TYPE_CHECKING:
    from sqlalchemy import Engine
    from sqlalchemy.orm import Session


@get(path="/sqlalchemy-app", sync_to_thread=True)
def async_sqlalchemy_init(db_session: Session, db_engine: Engine) -> str:
    """Interact with SQLAlchemy engine and session."""
    one = db_session.execute(text("SELECT 1")).scalar_one()

    with db_engine.connect() as conn:
        two = conn.execute(text("SELECT 2")).scalar_one()

    return f"{one} {two}"


sqlalchemy_config = SQLAlchemySyncConfig(connection_string="sqlite:///test.sqlite")

app = Litestar(
    route_handlers=[async_sqlalchemy_init],
    plugins=[SQLAlchemyInitPlugin(config=sqlalchemy_config)],
)



from __future__ import annotations

from dataclasses import dataclass

from typing_extensions import Annotated

from litestar import Litestar, post
from litestar.dto import DataclassDTO, DTOConfig


@dataclass
class User:
    id: str


UserDTO = DataclassDTO[Annotated[User, DTOConfig(forbid_unknown_fields=True)]]


@post("/users", dto=UserDTO)
async def create_user(data: User) -> User:
    return data


app = Litestar([create_user])


# run: /users -H "Content-Type: application/json" -d '{"id": "1", "name": "Peter"}'



from __future__ import annotations

import argparse
import importlib.metadata
import json
import os
import shutil
import subprocess
from contextlib import contextmanager
from pathlib import Path
from typing import TypedDict

REDIRECT_TEMPLATE = """
<!DOCTYPE HTML>
<html lang="en-US">
    <head>
        <title>Page Redirection</title>
        <meta charset="UTF-8">
        <meta http-equiv="refresh" content="0; url={target}">
        <script type="text/javascript">window.location.href = "{target}"</script>
    </head>
    <body>
        You are being redirected. If this does not work, click <a href='{target}'>this link</a>
    </body>
</html>
"""

parser = argparse.ArgumentParser()
parser.add_argument("--version", required=False)
parser.add_argument("output")


class VersionSpec(TypedDict):
    versions: list[str]
    latest: str


@contextmanager
def checkout(branch: str) -> None:
    subprocess.run(["git", "checkout", branch], check=True)  # noqa: S603 S607
    yield
    subprocess.run(["git", "checkout", "-"], check=True)  # noqa: S607


def load_version_spec() -> VersionSpec:
    versions_file = Path("docs/_static/versions.json")
    if versions_file.exists():
        return json.loads(versions_file.read_text())
    return {"versions": [], "latest": ""}


def build(output_dir: str, version: str | None) -> None:
    if version is None:
        version = importlib.metadata.version("litestar").rsplit(".")[0]
    else:
        os.environ["_LITESTAR_DOCS_BUILD_VERSION"] = version

    subprocess.run(["make", "docs"], check=True)  # noqa: S607

    output_dir = Path(output_dir)
    output_dir.mkdir()
    output_dir.joinpath(".nojekyll").touch(exist_ok=True)

    version_spec = load_version_spec()
    is_latest = version == version_spec["latest"]

    docs_src_path = Path("docs/_build/html")

    output_dir.joinpath("index.html").write_text(REDIRECT_TEMPLATE.format(target="latest"))

    if is_latest:
        shutil.copytree(docs_src_path, output_dir / "latest", dirs_exist_ok=True)
    shutil.copytree(docs_src_path, output_dir / version, dirs_exist_ok=True)

    # copy existing versions into our output dir to preserve them when cleaning the branch
    with checkout("gh-pages"):
        for other_version in [*version_spec["versions"], "latest"]:
            other_version_path = Path(other_version)
            other_version_target_path = output_dir / other_version
            if other_version_path.exists() and not other_version_target_path.exists():
                shutil.copytree(other_version_path, other_version_target_path)


def main() -> None:
    args = parser.parse_args()
    build(output_dir=args.output, version=args.version)


if __name__ == "__main__":
    main()



import pytest

from litestar import Litestar
from litestar.config.csrf import CSRFConfig
from litestar.openapi.config import OpenAPIConfig
from litestar.openapi.plugins import RapidocRenderPlugin, ScalarRenderPlugin, SwaggerRenderPlugin
from litestar.testing import TestClient

rapidoc_fragment = ".addEventListener('before-try',"
swagger_fragment = "requestInterceptor:"


def test_rapidoc_csrf() -> None:
    app = Litestar(
        csrf_config=CSRFConfig(secret="litestar"),
        openapi_config=OpenAPIConfig(
            title="Litestar Example",
            version="0.0.1",
            render_plugins=[RapidocRenderPlugin()],
        ),
    )

    with TestClient(app=app) as client:
        resp = client.get("/schema/rapidoc")
        assert resp.status_code == 200
        assert resp.headers["content-type"] == "text/html; charset=utf-8"
        assert rapidoc_fragment in resp.text


def test_swagger_ui_csrf() -> None:
    app = Litestar(
        csrf_config=CSRFConfig(secret="litestar"),
        openapi_config=OpenAPIConfig(
            title="Litestar Example",
            version="0.0.1",
            render_plugins=[SwaggerRenderPlugin()],
        ),
    )

    with TestClient(app=app) as client:
        resp = client.get("/schema/swagger")
        assert resp.status_code == 200
        assert resp.headers["content-type"] == "text/html; charset=utf-8"
        assert swagger_fragment in resp.text


def test_plugins_csrf_httponly() -> None:
    app = Litestar(
        csrf_config=CSRFConfig(secret="litestar", cookie_httponly=True),
        openapi_config=OpenAPIConfig(
            title="Litestar Example",
            version="0.0.1",
            render_plugins=[RapidocRenderPlugin(), SwaggerRenderPlugin()],
        ),
    )

    with TestClient(app=app) as client:
        resp = client.get("/schema/rapidoc")
        assert resp.status_code == 200
        assert rapidoc_fragment not in resp.text

        resp = client.get("/schema/swagger")
        assert resp.status_code == 200
        assert swagger_fragment not in resp.text


@pytest.mark.parametrize(
    "scalar_config",
    [
        {"showSidebar": False},
    ],
)
@pytest.mark.parametrize(
    "expected_config_render",
    [
        "document.getElementById('api-reference').dataset.configuration = '{\"showSidebar\":false}'",
    ],
)
def test_openapi_scalar_options(scalar_config: dict, expected_config_render: str) -> None:
    app = Litestar(
        openapi_config=OpenAPIConfig(
            title="Litestar Example",
            version="0.0.1",
            render_plugins=[ScalarRenderPlugin(options=scalar_config)],
        )
    )

    with TestClient(app=app) as client:
        resp = client.get("/schema/scalar")
        assert resp.status_code == 200
        assert expected_config_render in resp.text



from litestar import Controller, Litestar, MediaType, Router, get
from litestar.datastructures import Cookie


class MyController(Controller):
    path = "/controller-path"
    response_cookies = [
        Cookie(
            key="controller-cookie",
            value="controller value",
            description="controller level cookie",
        )
    ]

    @get(
        path="/",
        response_cookies=[
            Cookie(
                key="local-cookie",
                value="local value",
                description="route handler level cookie",
            )
        ],
        media_type=MediaType.TEXT,
        sync_to_thread=False,
    )
    def my_route_handler(self) -> str:
        return "hello world"


router = Router(
    path="/router-path",
    route_handlers=[MyController],
    response_cookies=[Cookie(key="router-cookie", value="router value", description="router level cookie")],
)

app = Litestar(
    route_handlers=[router],
    response_cookies=[Cookie(key="app-cookie", value="app value", description="app level cookie")],
)



from litestar import Litestar, get
from litestar.config.cors import CORSConfig
from litestar.status_codes import HTTP_204_NO_CONTENT, HTTP_400_BAD_REQUEST
from litestar.testing import TestClient


@get("/headers-test")
async def headers_handler() -> str:
    return "Test Successful!"


cors_config = CORSConfig(
    allow_methods=["GET"],
    allow_origins=["https://allowed-origin.com"],
    allow_headers=["X-Custom-Header", "Content-Type"],
)
app = Litestar(route_handlers=[headers_handler], cors_config=cors_config)


def test_cors_with_specific_allowed_headers() -> None:
    with TestClient(app) as client:
        response = client.options(
            "/endpoint",
            headers={
                "Origin": "https://allowed-origin.com",
                "Access-Control-Request-Method": "GET",
                "Access-Control-Request-Headers": "X-Custom-Header, Content-Type",
            },
        )
        assert response.status_code == HTTP_204_NO_CONTENT
        assert "x-custom-header" in response.headers["access-control-allow-headers"]
        assert "content-type" in response.headers["access-control-allow-headers"]


def test_cors_with_unauthorized_headers() -> None:
    with TestClient(app) as client:
        response = client.options(
            "/endpoint",
            headers={
                "Origin": "https://allowed-origin.com",
                "Access-Control-Request-Method": "GET",
                "Access-Control-Request-Headers": "X-Not-Allowed-Header",
            },
        )
        assert response.status_code == HTTP_400_BAD_REQUEST
        assert (
            "access-control-allow-headers" not in response.headers
            or "x-not-allowed-header" not in response.headers.get("access-control-allow-headers", "")
        )



import pytest

from litestar.testing import TestClient
from litestar.testing.life_span_handler import LifeSpanHandler
from litestar.types import Receive, Scope, Send

pytestmark = pytest.mark.filterwarnings("default")


async def test_wait_startup_invalid_event() -> None:
    async def app(scope: Scope, receive: Receive, send: Send) -> None:
        await send({"type": "lifespan.startup.something_unexpected"})  # type: ignore[typeddict-item]

    with pytest.raises(RuntimeError, match="Received unexpected ASGI message type"):
        with LifeSpanHandler(TestClient(app)):
            pass


async def test_wait_shutdown_invalid_event() -> None:
    async def app(scope: Scope, receive: Receive, send: Send) -> None:
        await send({"type": "lifespan.startup.complete"})  # type: ignore[typeddict-item]
        await send({"type": "lifespan.shutdown.something_unexpected"})  # type: ignore[typeddict-item]

    with LifeSpanHandler(TestClient(app)) as handler:
        with pytest.raises(RuntimeError, match="Received unexpected ASGI message type"):
            await handler.wait_shutdown()


async def test_implicit_startup() -> None:
    async def app(scope: Scope, receive: Receive, send: Send) -> None:
        await send({"type": "lifespan.startup.complete"})  # type: ignore[typeddict-item]
        await send({"type": "lifespan.shutdown.complete"})  # type: ignore[typeddict-item]

    with pytest.warns(DeprecationWarning):
        handler = LifeSpanHandler(TestClient(app))
        await handler.wait_shutdown()
        handler.close()



from litestar.app import Litestar
from litestar.connection import Request, WebSocket
from litestar.controller import Controller
from litestar.enums import HttpMethod, MediaType
from litestar.handlers import (
    asgi,
    delete,
    get,
    head,
    patch,
    post,
    put,
    route,
    websocket,
    websocket_listener,
    websocket_stream,
)
from litestar.response import Response
from litestar.router import Router
from litestar.utils.version import get_version

__version__ = get_version()


__all__ = (
    "Controller",
    "HttpMethod",
    "Litestar",
    "MediaType",
    "Request",
    "Response",
    "Router",
    "WebSocket",
    "__version__",
    "asgi",
    "delete",
    "get",
    "head",
    "patch",
    "post",
    "put",
    "route",
    "websocket",
    "websocket_listener",
    "websocket_stream",
)



import dataclasses
from typing import TYPE_CHECKING, Any, List, Optional, Type, cast
from uuid import UUID

import pytest
from typing_extensions import Annotated, NewType

from litestar import Controller, Litestar, Router, get
from litestar._openapi.datastructures import OpenAPIContext
from litestar._openapi.parameters import ParameterFactory
from litestar._openapi.schema_generation.examples import ExampleFactory
from litestar._openapi.typescript_converter.schema_parsing import is_schema_value
from litestar.di import Provide
from litestar.enums import ParamType
from litestar.exceptions import ImproperlyConfiguredException
from litestar.handlers import HTTPRouteHandler
from litestar.openapi import OpenAPIConfig
from litestar.openapi.spec import Example, OpenAPI, Reference, Schema
from litestar.openapi.spec.enums import OpenAPIType
from litestar.params import Dependency, Parameter
from litestar.routes import BaseRoute
from litestar.testing import create_test_client
from litestar.utils import find_index
from tests.unit.test_openapi.utils import Gender, LuckyNumber

if TYPE_CHECKING:
    from litestar.openapi.spec.parameter import Parameter as OpenAPIParameter


def create_factory(route: BaseRoute, handler: HTTPRouteHandler) -> ParameterFactory:
    return ParameterFactory(
        OpenAPIContext(
            openapi_config=OpenAPIConfig(title="Test API", version="1.0.0", create_examples=True), plugins=[]
        ),
        route_handler=handler,
        path_parameters=route.path_parameters,
    )


def _create_parameters(app: Litestar, path: str) -> List["OpenAPIParameter"]:
    index = find_index(app.routes, lambda x: x.path_format == path)
    route = app.routes[index]
    route_handler = route.route_handler_map["GET"][0]  # type: ignore[union-attr]
    handler = route_handler.fn
    assert callable(handler)
    return create_factory(route, route_handler).create_parameters_for_handler()


def test_create_parameters(person_controller: Type[Controller]) -> None:
    ExampleFactory.seed_random(10)

    parameters = _create_parameters(app=Litestar(route_handlers=[person_controller]), path="/{service_id}/person")
    assert len(parameters) == 10
    service_id, page, name, page_size, from_date, to_date, gender, lucky_number, secret_header, cookie_value = tuple(
        parameters
    )

    assert service_id.name == "service_id"
    assert service_id.param_in == ParamType.PATH
    assert is_schema_value(service_id.schema)
    assert service_id.schema.type == OpenAPIType.INTEGER
    assert service_id.required
    assert service_id.schema.examples

    assert page.param_in == ParamType.QUERY
    assert page.name == "page"
    assert is_schema_value(page.schema)
    assert page.schema.type == OpenAPIType.INTEGER
    assert page.required
    assert page.schema.examples

    assert page_size.param_in == ParamType.QUERY
    assert page_size.name == "pageSize"
    assert is_schema_value(page_size.schema)
    assert page_size.schema.type == OpenAPIType.INTEGER
    assert page_size.required
    assert page_size.description == "Page Size Description"
    assert page_size.examples
    assert page_size.schema.examples == [1]

    assert name.param_in == ParamType.QUERY
    assert name.name == "name"
    assert is_schema_value(name.schema)
    assert name.schema.one_of
    assert len(name.schema.one_of) == 3
    assert not name.required
    assert name.schema.examples

    assert from_date.param_in == ParamType.QUERY
    assert from_date.name == "from_date"
    assert is_schema_value(from_date.schema)
    assert from_date.schema.one_of
    assert len(from_date.schema.one_of) == 4
    assert not from_date.required
    assert from_date.schema.examples

    assert to_date.param_in == ParamType.QUERY
    assert to_date.name == "to_date"
    assert is_schema_value(to_date.schema)
    assert to_date.schema.one_of
    assert len(to_date.schema.one_of) == 4
    assert not to_date.required
    assert to_date.schema.examples

    assert gender.param_in == ParamType.QUERY
    assert gender.name == "gender"
    assert is_schema_value(gender.schema)
    assert gender.schema == Schema(
        one_of=[
            Reference(ref="#/components/schemas/tests_unit_test_openapi_utils_Gender"),
            Schema(
                type=OpenAPIType.ARRAY,
                items=Reference(ref="#/components/schemas/tests_unit_test_openapi_utils_Gender"),
                examples=[[Gender.FEMALE]],
            ),
            Schema(type=OpenAPIType.NULL),
        ],
        examples=[Gender.MALE, [Gender.MALE, Gender.OTHER]],
    )
    assert not gender.required

    assert secret_header.param_in == ParamType.HEADER
    assert is_schema_value(secret_header.schema)
    assert secret_header.schema.type == OpenAPIType.STRING
    assert secret_header.required
    assert secret_header.schema.examples

    assert cookie_value.param_in == ParamType.COOKIE
    assert is_schema_value(cookie_value.schema)
    assert cookie_value.schema.type == OpenAPIType.INTEGER
    assert cookie_value.required
    assert cookie_value.schema.examples

    assert lucky_number.param_in == ParamType.QUERY
    assert lucky_number.name == "lucky_number"
    assert is_schema_value(lucky_number.schema)
    assert lucky_number.schema == Schema(
        one_of=[
            Reference(ref="#/components/schemas/tests_unit_test_openapi_utils_LuckyNumber"),
            Schema(type=OpenAPIType.NULL),
        ],
        examples=[LuckyNumber.SEVEN],
    )
    assert not lucky_number.required


def test_deduplication_for_param_where_key_and_type_are_equal() -> None:
    class BaseDep:
        def __init__(self, query_param: str) -> None: ...

    class ADep(BaseDep): ...

    class BDep(BaseDep): ...

    async def c_dep(other_param: float) -> float:
        return other_param

    async def d_dep(other_param: float) -> float:
        return other_param

    @get(
        "/test",
        dependencies={
            "a": Provide(ADep, sync_to_thread=False),
            "b": Provide(BDep, sync_to_thread=False),
            "c": Provide(c_dep),
            "d": Provide(d_dep),
        },
    )
    def handler(a: ADep, b: BDep, c: float, d: float) -> str:
        return "OK"

    app = Litestar(route_handlers=[handler])
    assert isinstance(app.openapi_schema, OpenAPI)
    open_api_path_item = app.openapi_schema.paths["/test"]  # type: ignore[index]
    open_api_parameters = open_api_path_item.get.parameters  # type: ignore[union-attr]
    assert len(open_api_parameters) == 2  # type: ignore[arg-type]
    assert {p.name for p in open_api_parameters} == {"query_param", "other_param"}  # type: ignore[union-attr]


def test_raise_for_multiple_parameters_of_same_name_and_differing_types() -> None:
    async def a_dep(query_param: int) -> int:
        return query_param

    async def b_dep(query_param: str) -> int:
        return 1

    @get("/test", dependencies={"a": Provide(a_dep), "b": Provide(b_dep)})
    def handler(a: int, b: int) -> str:
        return "OK"

    app = Litestar(route_handlers=[handler])

    with pytest.raises(ImproperlyConfiguredException):
        app.openapi_schema


def test_dependency_params_in_docs_if_dependency_provided() -> None:
    async def produce_dep(param: str) -> int:
        return 13

    @get(dependencies={"dep": Provide(produce_dep)})
    def handler(dep: Optional[int] = Dependency()) -> None:
        return None

    app = Litestar(route_handlers=[handler])
    param_name_set = {p.name for p in cast("OpenAPI", app.openapi_schema).paths["/"].get.parameters}  # type: ignore[index, redundant-cast, union-attr]
    assert "dep" not in param_name_set
    assert "param" in param_name_set


def test_dependency_not_in_doc_params_if_not_provided() -> None:
    @get()
    def handler(dep: Optional[int] = Dependency()) -> None:
        return None

    app = Litestar(route_handlers=[handler])
    assert cast("OpenAPI", app.openapi_schema).paths["/"].get.parameters is None  # type: ignore[index, redundant-cast, union-attr]


def test_non_dependency_in_doc_params_if_not_provided() -> None:
    @get()
    def handler(param: Optional[int]) -> None:
        return None

    app = Litestar(route_handlers=[handler])
    param_name_set = {p.name for p in cast("OpenAPI", app.openapi_schema).paths["/"].get.parameters}  # type: ignore[index, redundant-cast, union-attr]
    assert "param" in param_name_set


def test_layered_parameters() -> None:
    class MyController(Controller):
        path = "/controller"
        parameters = {
            "controller1": Parameter(lt=100),
            "controller2": Parameter(str, query="controller3"),
        }

        @get("/{local:int}")
        def my_handler(
            self,
            local: int,
            controller1: int,
            router1: str,
            router2: float,
            app1: str,
            app2: List[str],
            controller2: float = Parameter(float, ge=5.0),
        ) -> dict:
            return {}

    router = Router(
        path="/router",
        route_handlers=[MyController],
        parameters={
            "router1": Parameter(str, pattern="^[a-zA-Z]$"),
            "router2": Parameter(float, multiple_of=5.0, header="router3"),
        },
    )

    parameters = _create_parameters(
        app=Litestar(
            route_handlers=[router],
            parameters={
                "app1": Parameter(str, cookie="app4"),
                "app2": Parameter(List[str], min_items=2),
                "app3": Parameter(bool, required=False),
            },
        ),
        path="/router/controller/{local}",
    )
    local, app3, controller1, router1, router3, app4, app2, controller3 = tuple(parameters)

    assert app4.param_in == ParamType.COOKIE
    assert app4.schema.type == OpenAPIType.STRING  # type: ignore[union-attr]
    assert app4.required
    assert app4.schema.examples  # type: ignore[union-attr]

    assert app2.param_in == ParamType.QUERY
    assert app2.schema.type == OpenAPIType.ARRAY  # type: ignore[union-attr]
    assert app2.required
    assert app2.schema.examples  # type: ignore[union-attr]

    assert app3.param_in == ParamType.QUERY
    assert app3.schema.type == OpenAPIType.BOOLEAN  # type: ignore[union-attr]
    assert not app3.required
    assert app3.schema.examples  # type: ignore[union-attr]

    assert router1.param_in == ParamType.QUERY
    assert router1.schema.type == OpenAPIType.STRING  # type: ignore[union-attr]
    assert router1.required
    assert router1.schema.pattern == "^[a-zA-Z]$"  # type: ignore[union-attr]
    assert router1.schema.examples  # type: ignore[union-attr]

    assert router3.param_in == ParamType.HEADER
    assert router3.schema.type == OpenAPIType.NUMBER  # type: ignore[union-attr]
    assert router3.required
    assert router3.schema.multiple_of == 5.0  # type: ignore[union-attr]
    assert router3.schema.examples  # type: ignore[union-attr]

    assert controller1.param_in == ParamType.QUERY
    assert controller1.schema.type == OpenAPIType.INTEGER  # type: ignore[union-attr]
    assert controller1.required
    assert controller1.schema.exclusive_maximum == 100.0  # type: ignore[union-attr]
    assert controller1.schema.examples  # type: ignore[union-attr]

    assert controller3.param_in == ParamType.QUERY
    assert controller3.schema.type == OpenAPIType.NUMBER  # type: ignore[union-attr]
    assert controller3.required
    assert controller3.schema.minimum == 5.0  # type: ignore[union-attr]
    assert controller3.schema.examples  # type: ignore[union-attr]

    assert local.param_in == ParamType.PATH
    assert local.schema.type == OpenAPIType.INTEGER  # type: ignore[union-attr]
    assert local.required
    assert local.schema.examples  # type: ignore[union-attr]


def test_parameter_examples() -> None:
    @get(path="/")
    async def index(
        text: Annotated[str, Parameter(examples=[Example(value="example value", summary="example summary")])],
    ) -> str:
        return text

    with create_test_client(
        route_handlers=[index], openapi_config=OpenAPIConfig(title="Test API", version="1.0.0")
    ) as client:
        response = client.get("/schema/openapi.json")
        assert response.json()["paths"]["/"]["get"]["parameters"][0]["examples"] == {
            "text-example-1": {"summary": "example summary", "value": "example value"}
        }


def test_parameter_schema_extra() -> None:
    @get()
    async def handler(
        query1: Annotated[
            str,
            Parameter(
                schema_extra={
                    "schema_not": Schema(
                        any_of=[
                            Schema(type=OpenAPIType.STRING, pattern=r"^somePrefix:.*$"),
                            Schema(type=OpenAPIType.STRING, enum=["denied", "values"]),
                        ]
                    ),
                }
            ),
        ],
        query2: Annotated[
            Gender,
            Parameter(description="gender description", schema_extra={"format": "foo"}, schema_component_key="q2"),
        ],
        query3: Annotated[Gender, Parameter(schema_extra={"format": "bar"}, schema_component_key="q3")],
    ) -> Any:
        return query1

    @get()
    async def error_handler(query1: Annotated[str, Parameter(schema_extra={"invalid": "dummy"})]) -> Any:
        return query1

    # Success
    app = Litestar([handler])
    schema = app.openapi_schema.to_schema()
    assert schema["paths"]["/"]["get"]["parameters"][0]["schema"]["not"] == {
        "anyOf": [
            {"type": "string", "pattern": r"^somePrefix:.*$"},
            {"type": "string", "enum": ["denied", "values"]},
        ]
    }
    assert schema["paths"]["/"]["get"]["parameters"][1]["schema"]["$ref"] == "#/components/schemas/q2"
    assert schema["paths"]["/"]["get"]["parameters"][1]["description"] == "gender description"
    assert schema["components"]["schemas"]["q2"]["format"] == "foo"
    assert schema["paths"]["/"]["get"]["parameters"][2]["schema"]["$ref"] == "#/components/schemas/q3"
    assert schema["paths"]["/"]["get"]["parameters"][2]["description"] == Gender.__doc__
    assert schema["components"]["schemas"]["q3"]["format"] == "bar"

    # Attempt to pass invalid key
    app = Litestar([error_handler])
    with pytest.raises(ValueError) as e:
        app.openapi_schema
    assert str(e.value).startswith("`schema_extra` declares key")


def test_uuid_path_description_generation() -> None:
    # https://github.com/litestar-org/litestar/issues/2967
    @get("str/{id:str}")
    async def str_path(id: Annotated[str, Parameter(description="String ID")]) -> str:
        return id

    @get("uuid/{id:uuid}")
    async def uuid_path(id: Annotated[UUID, Parameter(description="UUID ID")]) -> UUID:
        return id

    with create_test_client(
        [str_path, uuid_path], openapi_config=OpenAPIConfig(title="Test API", version="1.0.0")
    ) as client:
        response = client.get("/schema/openapi.json")
        assert response.json()["paths"]["/str/{id}"]["get"]["parameters"][0]["description"] == "String ID"
        assert response.json()["paths"]["/uuid/{id}"]["get"]["parameters"][0]["description"] == "UUID ID"


def test_unwrap_new_type() -> None:
    FancyString = NewType("FancyString", str)

    @get("/{path_param:str}")
    async def handler(
        param: FancyString,
        optional_param: Optional[FancyString],
        path_param: FancyString,
    ) -> FancyString:
        return FancyString("")

    app = Litestar([handler])
    assert app.openapi_schema.paths["/{path_param}"].get.parameters[0].schema.type == OpenAPIType.STRING  # type: ignore[index, union-attr]
    assert app.openapi_schema.paths["/{path_param}"].get.parameters[1].schema.one_of == [  # type: ignore[index, union-attr]
        Schema(type=OpenAPIType.STRING),
        Schema(type=OpenAPIType.NULL),
    ]
    assert app.openapi_schema.paths["/{path_param}"].get.parameters[2].schema.type == OpenAPIType.STRING  # type: ignore[index, union-attr]
    assert (
        app.openapi_schema.paths["/{path_param}"].get.responses["200"].content["application/json"].schema.type  # type: ignore[index, union-attr]
        == OpenAPIType.STRING
    )


def test_unwrap_nested_new_type() -> None:
    FancyString = NewType("FancyString", str)
    FancierString = NewType("FancierString", FancyString)  # pyright: ignore

    @get("/")
    async def handler(
        param: FancierString,
    ) -> None:
        return None

    app = Litestar([handler])
    assert app.openapi_schema.paths["/"].get.parameters[0].schema.type == OpenAPIType.STRING  # type: ignore[index, union-attr]


def test_unwrap_annotated_new_type() -> None:
    FancyString = NewType("FancyString", str)

    @dataclasses.dataclass
    class TestModel:
        param: Annotated[FancyString, "foo"]

    @get("/")
    async def handler(
        param: TestModel,
    ) -> None:
        return None

    app = Litestar([handler])

    testmodel_schema_name = app.openapi_schema.paths["/"].get.parameters[0].schema.value  # type: ignore[index, union-attr]
    assert app.openapi_schema.components.schemas[testmodel_schema_name].properties["param"].type == OpenAPIType.STRING  # type: ignore[index, union-attr]


def test_query_param_only_properties() -> None:
    # https://github.com/litestar-org/litestar/issues/3908
    @get("/{path_param:str}")
    def handler(
        path_param: str,
        query_param: str,
        header_param: Annotated[str, Parameter(header="header_param")],
        cookie_param: Annotated[str, Parameter(cookie="cookie_param")],
    ) -> None:
        pass

    app = Litestar([handler])
    params = {p.name: p for p in app.openapi_schema.paths["/{path_param}"].get.parameters}  # type: ignore[union-attr, index]

    for key in ["path_param", "header_param", "cookie_param"]:
        schema = params[key].to_schema()
        assert "allowEmptyValue" not in schema
        assert "allowReserved" not in schema

    assert params["query_param"].to_schema() == {
        "name": "query_param",
        "in": "query",
        "schema": {"type": "string"},
        "required": True,
        "deprecated": False,
        "allowEmptyValue": False,
        "allowReserved": False,
    }


def test_not_included_in_schema_parameter() -> None:
    @get("/handler")
    async def handler(param: Annotated[str, Parameter(include_in_schema=False)]) -> None:
        pass

    with create_test_client(handler) as client:
        response = client.get("/schema/openapi.json")

        response_json = response.json()
        handler_schema = response_json["paths"]["/handler"]["get"]
        assert "parameters" not in handler_schema


def test_two_parameters_but_one_not_included_in_schema() -> None:
    @get("/handler")
    def handler(param1: str, param2: str = Parameter(include_in_schema=False)) -> None:
        pass

    with create_test_client(handler) as client:
        response = client.get("/schema/openapi.json")

        response_json = response.json()
        handler_schema = response_json["paths"]["/handler"]["get"]
        assert "parameters" in handler_schema

        parameter_names = {param["name"] for param in handler_schema["parameters"]}
        assert "param1" in parameter_names
        assert "param2" not in parameter_names



from dataclasses import dataclass

from litestar import Litestar, post
from litestar.dto import DataclassDTO


@dataclass
class Foo:
    this_will: str
    _this_will: str = "Mars"


@post("/", dto=DataclassDTO[Foo], sync_to_thread=False)
def handler(data: Foo) -> Foo:
    return data


app = Litestar(route_handlers=[handler])

# run: / -H "Content-Type: application/json" -d '{"bar":"stay","_baz":"go_away!"}'



from __future__ import annotations

from dataclasses import dataclass
from typing import List

from litestar import Litestar, get
from litestar.dto import DataclassDTO, DTOConfig


@dataclass
class Address:
    street: str
    city: str
    country: str


@dataclass
class Person:
    name: str
    age: int
    email: str
    address: Address
    children: List[Person]


class ReadDTO(DataclassDTO[Person]):
    config = DTOConfig(
        exclude={"email", "address.street", "children.0.email", "children.0.address"},
        rename_fields={"address": "location"},
    )


@get("/person/{name:str}", return_dto=ReadDTO, sync_to_thread=False)
def get_person(name: str) -> Person:
    # Your logic to retrieve the person goes here
    # For demonstration purposes, a placeholder Person instance is returned
    address = Address(street="123 Main St", city="Cityville", country="Countryland")
    child1 = Person(name="Child1", age=10, email="child1@example.com", address=address, children=[])
    child2 = Person(name="Child2", age=8, email="child2@example.com", address=address, children=[])
    return Person(
        name=name,
        age=30,
        email=f"email_of_{name}@example.com",
        address=address,
        children=[child1, child2],
    )


app = Litestar(route_handlers=[get_person])



from pathlib import Path

from litestar import Litestar
from litestar.contrib.mako import MakoTemplateEngine
from litestar.template.config import TemplateConfig

app = Litestar(
    route_handlers=[],
    template_config=TemplateConfig(
        directory=Path("templates"),
        engine=MakoTemplateEngine,
    ),
)



from __future__ import annotations

import sys
from typing import TYPE_CHECKING, Any, NamedTuple, get_args, get_origin

import typing_extensions

from litestar.plugins import OpenAPISchemaPlugin
from litestar.typing import FieldDefinition

if TYPE_CHECKING:
    from litestar._openapi.schema_generation import SchemaCreator
    from litestar.openapi.spec import Schema


class _TypedDictInfo(NamedTuple):
    required_keys: frozenset[str]
    optional_keys: frozenset[str]
    readonly_keys: frozenset[str]
    mutable_keys: frozenset[str]


def _get_typed_dict_info(tp: Any) -> _TypedDictInfo:
    if sys.version_info < (3, 9):
        return _make_typed_dict_info(tp)

    return _TypedDictInfo(
        required_keys=tp.__required_keys__,
        optional_keys=tp.__optional_keys__,
        readonly_keys=getattr(tp, "__readonly_keys__", frozenset()),
        mutable_keys=getattr(tp, "__mutable_keys__", frozenset()),
    )


# backport of the 3.9+ logic to extract TypedDict information
def _get_typeddict_qualifiers(annotation_type: Any) -> Any:
    while True:
        annotation_origin = get_origin(annotation_type)
        if annotation_origin is typing_extensions.Annotated:
            annotation_args = get_args(annotation_type)
            if annotation_args:
                annotation_type = annotation_args[0]
            else:
                break
        elif annotation_origin is typing_extensions.Required:
            yield typing_extensions.Required
            (annotation_type,) = get_args(annotation_type)
        elif annotation_origin is typing_extensions.NotRequired:
            yield typing_extensions.NotRequired
            (annotation_type,) = get_args(annotation_type)
        elif annotation_origin is typing_extensions.ReadOnly:
            yield typing_extensions.ReadOnly
            (annotation_type,) = get_args(annotation_type)
        else:
            break


def _make_typed_dict_info(tp: Any) -> _TypedDictInfo:
    required_keys: set[str] = set()
    optional_keys: set[str] = set()
    readonly_keys: set[str] = set()
    mutable_keys: set[str] = set()
    bases = tp.mro()

    annotations = {}
    own_annotations = getattr(tp, "__annotations__", {})

    for base in bases:
        annotations.update(base.__dict__.get("__annotations__", {}))

        base_required = base.__dict__.get("__required_keys__", set())
        required_keys |= base_required
        optional_keys -= base_required

        base_optional = base.__dict__.get("__optional_keys__", set())
        required_keys -= base_optional
        optional_keys |= base_optional

        readonly_keys.update(base.__dict__.get("__readonly_keys__", ()))
        mutable_keys.update(base.__dict__.get("__mutable_keys__", ()))

    annotations.update(own_annotations)
    for annotation_key, annotation_type in own_annotations.items():
        qualifiers = set(_get_typeddict_qualifiers(annotation_type))
        if typing_extensions.Required in qualifiers:
            is_required = True
        elif typing_extensions.NotRequired in qualifiers:
            is_required = False
        else:
            is_required = tp.__total__

        if is_required:
            required_keys.add(annotation_key)
            optional_keys.discard(annotation_key)
        else:
            optional_keys.add(annotation_key)
            required_keys.discard(annotation_key)

        if typing_extensions.ReadOnly in qualifiers:
            if annotation_key in mutable_keys:
                raise TypeError(f"Cannot override mutable key {annotation_key!r} with read-only key")
            readonly_keys.add(annotation_key)
        else:
            mutable_keys.add(annotation_key)
            readonly_keys.discard(annotation_key)

    return _TypedDictInfo(
        frozenset(required_keys),
        frozenset(optional_keys),
        frozenset(readonly_keys),
        frozenset(mutable_keys),
    )


class TypedDictSchemaPlugin(OpenAPISchemaPlugin):
    def is_plugin_supported_field(self, field_definition: FieldDefinition) -> bool:
        return field_definition.is_typeddict_type

    def to_openapi_schema(self, field_definition: FieldDefinition, schema_creator: SchemaCreator) -> Schema:
        type_hints = field_definition.get_type_hints(include_extras=True, resolve_generics=True)

        info = _get_typed_dict_info(field_definition.type_)

        return schema_creator.create_component_schema(
            field_definition,
            required=sorted(info.required_keys),
            property_fields={k: FieldDefinition.from_kwarg(v, k) for k, v in type_hints.items()},
        )
